"""
Manages the configuration of PhysBiblio, from saving to reading.

This file is part of the physbiblio package.
"""
import sys, ast, os
import logging
from appdirs import AppDirs

try:
	from physbiblio.databaseCore import physbiblioDBCore, physbiblioDBSub
	from physbiblio.tablesDef import profilesSettingsTable, searchesTable
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
	print(traceback.format_exc())
	raise

#these are the default parameter values, descriptions and types
configuration_params = [
{"name": "mainDatabaseName",
	"default": 'PBDATAphysbiblio.db',
	"description": 'Name of the database file',
	"special": None},
{"name": "loggingLevel",
	"default": 1,
	"description": 'How many messages to save in the log file (will have effects only after closing the application)',
	"special": 'int'},
{"name": "logFileName",
	"default": 'PBDATAparams.log',
	"description": 'Name of the log file',
	"special": None},
{"name": "pdfFolder",
	"default": 'PBDATApdf',
	"description": 'Folder where to save the PDF files',
	"special": None},
{"name": "pdfApplication",
	"default": '',
	"description": 'Application for opening PDF files (used only via command line)',
	"special": None},
{"name": "webApplication",
	"default": '',
	"description": 'Web browser (used only via command line)',
	"special": None},
{"name": "timeoutWebSearch",
	"default": 20.,
	"description": 'Timeout for the web queries',
	"special": 'float'},
{"name": "askBeforeExit",
	"default": False,
	"description": 'Confirm before exiting',
	"special": 'boolean'},
{"name": "defaultLimitBibtexs",
	"default": 100,
	"description": 'Number of bibtex entries in the initial view of the main table',
	"special": 'int'},
{"name": "defaultUpdateFrom",
	"default": 0,
	"description": 'Index of bibtex entries (firstdate ASC) from which I should start when using "Update bibtexs"',
	"special": 'int'},
{"name": "maxAuthorNames",
	"default": 3,
	"description": 'Max number of authors to be displayed in the main list',
	"special": 'int'},
{"name": "maxAuthorSave",
	"default": 6,
	"description": 'Max number of authors to be saved when adding info from arXiv',
	"special": 'int'},
{"name": "maxArxivResults",
	"default": 10,
	"description": 'Max number of entries per page when reading arxiv API results',
	"special": 'int'},
{"name": "fetchAbstract",
	"default": False,
	"description": 'Automatically fetch the abstract from arXiv if an arxiv number is present',
	"special": 'boolean'},
{"name": "defaultCategories",
	"default": [],
	"description": 'Default categories for imported bibtexs',
	"special": 'list'},
{"name": "bibListFontSize",
	"default": 9,
	"description": 'Font size in the list of bibtex entries and companion boxes',
	"special": 'float'},
{"name": "bibtexListColumns",
	"default": ["bibkey", "author", "title", "year", "firstdate", "pubdate", "doi", "arxiv", "isbn", "inspire"],
	"description": 'The columns to be shown in the entries list',
	"special": 'list'},
{"name": "maxSavedSearches",
	"default": 5,
	"description": 'Max number of automatically saved search/replace arguments',
	"special": 'int'},
]

loggingLevels = [
	"0 - errors",
	"1 - warnings",
	"2 - info",
	"3 - all"
]

config_paramOrder = [ p["name"] for p in configuration_params if p["name"] != "mainDatabaseName"]
config_defaults = {}
config_descriptions = {}
config_special = {}
for p in configuration_params:
	config_defaults[p["name"]] = p["default"]
	config_descriptions[p["name"]] = p["description"]
	config_special[p["name"]] = p["special"]

class globalDB(physbiblioDBCore):
	"""
	Class that manages the operations on the global DB: profiles and frequent searches/replaces
	"""
	def __init__(self, dbname, logger, datapath, info = True):
		"""
		Class constructor

		Parameters:
			dbname: the name of the database to open
			logger: the logger used for messages
			datapath: the path where to store database files
			info (boolean, default True): print some output messages
		"""
		self.dataPath = datapath
		physbiblioDBCore.__init__(self, dbname, logger, noOpen = True)
		self.openDB(info = info)

		self.cursExec("SELECT name FROM sqlite_master WHERE type='table';")
		tables = [name[0] for name in self.curs]
		if not all([a in tables for a in ["profiles", "searches"]]):
			self.createTables(tables)

		if self.countProfiles() == 0:
			self.createProfile()

	def createTables(self, existing):
		"""
		Create the profiles table
		"""
		if "profiles" not in existing:
			command = "CREATE TABLE profiles (\n"
			for el in profilesSettingsTable:
				command += " ".join(el) + ",\n"
			command += "CONSTRAINT unique_databasefile UNIQUE (databasefile)\n);"
			self.logger.info(command+"\n")
			if not self.connExec(command):
				self.logger.critical("Create profiles table failed")
				sys.exit(1)
			self.commit()
		if "searches" not in existing:
			command = "CREATE TABLE searches (\n"
			first = True
			for el in searchesTable:
				if first:
					first = False
				else:
					command += ",\n"
				command += " ".join(el)
			command += ");"
			self.logger.info(command + "\n")
			if not self.connExec(command):
				self.logger.critical("Create searches table failed")
				sys.exit(1)
			self.commit()
		return True

	def countProfiles(self):
		"""
		Obtain the number of profiles in the database

		Output:
			the number of profiles
		"""
		self.cursExec("SELECT Count(*) FROM profiles")
		return self.curs.fetchall()[0][0]

	def createProfile(self,
			name = "default",
			description = "",
			databasefile = None,
			oldCfg = ""):
		"""
		Create a new profile

		Parameters:
			name: the name of the profile
			description: a short description of the profile
			databasefile: the name of the database file which will contain data
			oldCfg: the name of the old .cfg file (backwards compatibility only)

		Output:
			True if successful, sys.exit(1) otherwise
		"""
		if databasefile is None:
			databasefile = os.path.join(self.dataPath, config_defaults["mainDatabaseName"].replace("PBDATA", ""))
		command = "INSERT into profiles (name, description, databasefile, oldCfg, isDefault, ord) " + \
			'values (:name, :description, :databasefile, :oldCfg, :isDefault, :ord)'
		data = {
			"name": name,
			"description": description,
			"databasefile": databasefile,
			"oldCfg": oldCfg,
			"isDefault": 1 if self.countProfiles() == 0 else 0,
			"ord": 100}
		self.logger.info("%s\n%s"%(command, data))
		if not self.connExec(command, data):
			self.logger.exception("Cannot insert profile")
			sys.exit(1)
		self.commit(verbose = False)
		return True

	def updateProfileField(self, identifier, field, value, identifierField = "name"):
		"""
		Update a field of an existing profile

		Parameters:
			identifier: the identifier of the profile. It may be 1 if `identifierField` == "isDefault", the name of the profile or the database filename if `identifierField` == "name" or "databasefile", respectively.
			field: the name of the field to update.
			value: the new vallue of the field
			identifierField: "name", "databasefile" or "isDefault". It must be "databasefile" if you want to change the dprofile name and "name" if you want to change the databasefile (discouraged, however)
		
		Output:
			True if successful, False otherwise
		"""
		if (field == "databasefile" and identifierField != "name") or \
				(field == "name" and identifierField != "databasefile") or \
				(identifierField == "isDefault" and identifier != 1) or \
				identifierField not in ["name", "databasefile", "isDefault"] or \
				field not in [e[0] for e in profilesSettingsTable]:
			self.logger.error("Invalid field or identifierField: %s, %s"%(field, identifierField))
			return False
		command = "update profiles set %s = :val "%field + \
			' where %s = :iden\n'%identifierField
		data = {
			"val": value,
			"iden": identifier,
			}
		self.logger.debug("%s\n%s"%(command, data))
		if not self.connExec(command, data):
			self.logger.error("Cannot insert profile")
			return False
		self.commit(verbose = False)
		return True

	def deleteProfile(self, name):
		"""
		Delete a profile given the name

		Parameters:
			name: the profile name

		Output:
			True if successful, False otherwise
		"""
		if name.strip() == "":
			self.logger.error("You must provide the profile name!")
			return False
		command = "delete from profiles where name = :name\n"
		self.logger.debug("%s\n%s"%(command, {"name": name}))
		if not self.connExec(command, {"name": name}):
			self.logger.error("Cannot delete profile")
			return False
		self.commit(verbose = False)
		return True

	def getProfiles(self):
		"""
		Get the list of profiles

		Output:
			the list of `sqlite3.Row` objects
		"""
		self.cursExec("SELECT * FROM profiles order by ord ASC, name ASC\n")
		return self.curs.fetchall()

	def getProfile(self, name = "", filename = ""):
		"""
		Get a profile given its name or the name of the database file. Note that only one of the two may be given

		Parameters:
			name: the name of the profile
			filename: the database filename of the profile

		Output:
			the `sqlite3.Row` object
		"""
		if name.strip() == "" and filename.strip() == "":
			self.logger.warning("You should specify the name or the filename associated with the profile")
			return {}
		if name.strip() != "" and filename.strip() != "":
			self.logger.warning("You should specify only the name or only the filename associated with the profile")
			return {}
		self.cursExec("SELECT * FROM profiles WHERE name = :name or databasefile = :file\n",
			{"name": name, "file": filename})
		return self.curs.fetchall()[0]

	def getProfileOrder(self):
		"""
		Obtain the order of profiles

		Output:
			a list with the names of the ordered profiles
		"""
		if self.countProfiles() == 0:
			self.createProfile()
			self.setDefaultProfile("default")
		self.cursExec("SELECT * FROM profiles order by ord ASC, name ASC\n")
		return [e["name"] for e in self.curs.fetchall()]

	def setProfileOrder(self, order = []):
		"""
		Set the new order of profiles

		Parameters:
			order: the ordered list of profile names

		Output:
			True if successful, False otherwise
		"""
		if order == []:
			self.logger.warning("No order given!")
			return False
		if sorted(order) != sorted([e["name"] for e in self.getProfiles()]):
			self.logger.info(sorted(order))
			self.logger.info(sorted([e["name"] for e in self.getProfiles()]))
			self.logger.warning("List of profile names does not match existing profiles!")
			return False
		failed = False
		for ix, profName in enumerate(order):
			if not self.connExec("update profiles set ord=:ord where name=:name\n", {"name": profName, "ord": ix}):
				failed = True
		if not failed:
			self.commit(verbose = False)
		else:
			self.logger.error("Something went wrong when setting new profile order. Undoing...")
			self.undo(verbose = False)
		return (not failed)

	def getDefaultProfile(self):
		"""
		Obtain the name of the default profile

		Output:
			the `sqlite3.Row` objects
		"""
		if self.countProfiles() == 0:
			self.createProfile()
			self.setDefaultProfile("default")
		self.cursExec("SELECT * FROM profiles WHERE isDefault = 1\n")
		return [e["name"] for e in self.curs.fetchall()][0]

	def setDefaultProfile(self, name = None):
		"""
		Set the new default profile

		Parameters:
			name: the profile name

		Output:
			True if successful, False otherwise
		"""
		if name is None or name.strip() == "":
			self.logger.warning("No name given!")
			return False
		self.cursExec("SELECT * FROM profiles WHERE name = :name\n", {"name": name})
		if len(self.curs.fetchall()) == 1:
			if self.connExec("update profiles set isDefault=0 where 1\n") and \
					self.connExec("update profiles set isDefault=1 where name = :name\n", {"name": name}):
				self.commit(verbose = False)
				return True
			else:
				self.logger.error("Something went wrong when setting new default profile. Undoing...")
				self.undo(verbose = False)
				return False
		else:
			self.logger.warning("No profiles with the given name!")
			return False

	def countSearches(self):
		"""
		Obtain the number of searches in the table

		Output:
			the number of searches
		"""
		self.cursExec("SELECT Count(*) FROM searches")
		return self.curs.fetchall()[0][0]

	def insertSearch(self, name = "", count = 0, searchDict = {}, replaceFields = [], manual = False, replacement = False, limit = 0, offset = 0):
		"""
		Insert a new search/replace

		Parameters:
			name: the config name
			count: the order in the cronology or in the menu
			searchDict: the dictionary which is meant to be passed to fetchByDict
			replaceFields: the replace fields used in searchAndReplace
			manual (boolean, default False): manually saved entry
			replacement (boolean, default False): replace or simple search
			limit: the number of requested entries
			offset: the offset

		Output:
			the output of self.connExec
		"""
		if limit == 0:
			limit = pbConfig.params["defaultLimitBibtexs"]
		output = self.connExec("INSERT into searches (name, count, searchDict, limitNum, offsetNum, replaceFields, manual, isReplace) values (:name, :count, :searchDict, :limit, :offset, :replaceFields, :manual, :isReplace)\n",
			{"name": name,
			"count": count,
			"searchDict": "%s"%searchDict,
			"limit": limit,
			"offset": offset,
			"replaceFields": "%s"%replaceFields,
			"manual": 1 if manual else 0,
			"isReplace": 1 if replacement else 0,
			})
		self.commit()
		return output

	def deleteSearch(self, idS):
		"""
		Delete a search/replace given the id.

		Parameters:
			idS: the unique identifier

		Output:
			the output of cursExec
		"""
		output = self.cursExec("delete from searches where idS=?\n", (idS, ))
		self.commit()
		return output

	def getAllSearches(self):
		"""
		Get all the searches

		Output:
			the list of `sqlite3.Row` objects with all the searches in the database
		"""
		self.cursExec("select * from searches order by count asc\n")
		return self.curs.fetchall()

	def getSearchByID(self, idS):
		"""
		Get a search given its name

		Parameters:
			idS: the name of the search/replace

		Output:
			the list (len = 1) of `sqlite3.Row` objects with all the matching searches
		"""
		self.cursExec("select * from searches where idS=?\n", (idS, ))
		return self.curs.fetchall()

	def getSearchByName(self, name):
		"""
		Get a search given its name

		Parameters:
			name: the name of the search/replace

		Output:
			the list of `sqlite3.Row` objects with all the matching searches
		"""
		self.cursExec("select * from searches where name=?\n", (name, ))
		return self.curs.fetchall()

	def getSearchList(self, manual = False, replacement = False):
		"""
		Get searches or replaces which were not manually saved

		Parameters:
			manual (boolean, default False): manually saved entry
			replacement (boolean, default False): replace or simple search

		Output:
			the list of `sqlite3.Row` objects with all the matching searches
		"""
		self.cursExec("select * from searches where manual=? and isReplace=? order by count ASC\n", (
			1 if manual else 0,
			1 if replacement else 0))
		return self.curs.fetchall()

	def updateSearchOrder(self, replacement = False):
		"""
		Update the cronology order for searches or replaces which were not manually saved

		Parameters:
			replacement (boolean, default False): replace or simple search

		Output:
			True if successfull, False if some sum failed
		"""
		self.cursExec("select * from searches where manual=? and isReplace=?\n", (0, 1 if replacement else 0))
		for e in self.curs.fetchall():
			if e["count"] + 1 >= pbConfig.params["maxSavedSearches"]:
				self.deleteSearch(e["idS"])
			if not self.connExec("update searches set count = :count where idS=:idS\n",
					{"idS": e["idS"], "count": e["count"] + 1}):
				self.undo()
				return False
		self.commit()
		return True

class configurationDB(physbiblioDBSub):
	"""
	Subclass that manages the functions for the categories.
	"""
	def count(self):
		"""
		Obtain the number of settings in the table

		Output:
			the number of settings
		"""
		self.cursExec("SELECT Count(*) FROM settings")
		return self.curs.fetchall()[0][0]

	def insert(self, name, value):
		"""
		Insert a new setting. If already existing, update it

		Parameters:
			name: the config name
			value: the content of the setting

		Output:
			the output of self.connExec
		"""
		self.cursExec("select * from settings where name=?\n", (name,))
		if len(self.curs.fetchall()) > 0:
			self.mainDB.logger.info("An entry with the same name is already present. Updating it")
			return self.update(name, value)
		else:
			return self.connExec("INSERT into settings (name, value) values (:name, :value)\n",
				{"name": name, "value": value})

	def update(self, name, value):
		"""
		Update an existing setting

		Parameters:
			name: the config name
			value: the content of the setting

		Output:
			the output of self.connExec
		"""
		self.cursExec("select * from settings where name=?\n", (name,))
		if len(self.curs.fetchall()) == 0:
			self.mainDB.logger.info("No settings found with this name. Inserting it")
			return self.insert(name, value)
		else:
			return self.connExec("update settings set value = :value where name = :name\n",
				{"name": name, "value": value})

	def delete(self, name):
		"""
		Delete a setting.

		Parameters:
			name: the setting name

		Output:
			the output of cursExec
		"""
		return self.cursExec("delete from settings where name=?\n", (name, ))

	def getAll(self):
		"""
		Get all the settings

		Output:
			the list of `sqlite3.Row` objects with all the settings in the database
		"""
		self.cursExec("select * from settings\n")
		return self.curs.fetchall()

	def getByName(self, name):
		"""
		Get a setting given its name

		Parameters:
			name: the name of the setting

		Output:
			the list (len = 1) of `sqlite3.Row` objects with all the matching settings
		"""
		self.cursExec("select * from settings where name=?\n", (name, ))
		return self.curs.fetchall()

class ConfigVars():
	"""
	Contains all the common settings, the information on the profiles and their configuration.
	"""
	def __init__(self, profileFileName = "profiles.db"):
		"""
		Initialize the configuration.
		Check the profiles first, then load the default profile and its configuration.
		"""
		#needed because the main logger will be loaded later!
		logging.basicConfig(format = '[%(module)s.%(funcName)s] %(message)s', level = logging.INFO)
		self.prepareLogger("physbibliolog")
		self.defaultDirs = AppDirs("PhysBiblio")

		self.configPath = self.defaultDirs.user_config_dir + os.sep
		self.dataPath = self.defaultDirs.user_data_dir + os.sep
		self.logger.info("Default configuration path: %s"%self.configPath)
		if not os.path.exists(self.configPath):
			os.makedirs(self.configPath)
		self.logger.info("Default data path: %s"%self.dataPath)
		if not os.path.exists(self.dataPath):
			os.makedirs(self.dataPath)

		self.paramOrder = config_paramOrder
		self.specialTypes = config_special
		self.defaultsParams = config_defaults
		self.loggingLevels = loggingLevels
		self.params = {}
		for k, v in config_defaults.items():
			if type(v) is str and "PBDATA" in v:
				v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
				config_defaults[k] = v
			self.params[k] = v
		self.descriptions = config_descriptions

		self.configProfilesFile = os.path.join(self.configPath, "profiles.dat")
		self.globalDbFile = os.path.join(self.configPath, profileFileName)
		self.globalDb = globalDB(self.globalDbFile, self.logger, self.dataPath, info = False)

		self.checkOldProfiles()
		self.loadProfiles()

		#some urls
		self.arxivUrl = "https://arxiv.org"
		self.doiUrl = "https://doi.org/"
		self.inspireRecord = "https://inspirehep.net/record/"
		self.inspireSearchBase = "https://inspirehep.net/search"

	def prepareLogger(self, string):
		"""
		Replace the logger used by this module

		Parameters:
			string: the string used in getLogger
		"""
		self.loggerString = string
		self.logger = logging.getLogger(self.loggerString)

	def loadProfiles(self):
		"""
		Load the information from the profile database
		"""
		try:
			self.defaultProfileName, self.profiles, self.profileOrder = self.readProfiles()
		except (IOError, ValueError, SyntaxError) as e:
			self.logger.warning(e)
			self.globalDb.createProfile()

	def reloadProfiles(self, useProfile = None):
		"""
		Load the information from the profile database, reset the default and current profile and the settings

		Parameters:
			useProfile (optional): the name of the profile to be used instead of the default one
		"""
		self.loadProfiles()

		self.currentProfileName = self.defaultProfileName if useProfile is None else useProfile
		try:
			self.currentProfile = self.profiles[self.currentProfileName]
		except KeyError:
			self.logger.critical("The profile '%s' does not exist! Back to the default one ('%s')"%(useProfile, self.defaultProfileName))
			self.currentProfileName = self.defaultProfileName
			self.currentProfile = self.profiles[self.currentProfileName]
		self.currentDatabase = self.currentProfile["db"]
		self.logger.info("Starting with profile '%s', database: %s"%(self.currentProfileName, self.currentDatabase))

		self.readConfig()

	def checkOldProfiles(self):
		"""
		Intended for backwards compatibility.
		Check if there is a profiles.dat file in the self.configPath folder.
		If yes, move it and the related information into the databases.
		"""
		if os.path.isfile(self.configProfilesFile):
			self.logger.info("Moving info from profiles.dat into the profiles.db")
			for k in self.globalDb.getProfileOrder():
				self.globalDb.deleteProfile(k)
			defProf, profiles, profileOrder = self.oldReadProfiles()
			for k in profiles.keys():
				self.oldReInit(k, profiles[k])
				tempDb = physbiblioDBCore(self.params["mainDatabaseName"], self.logger)
				configDb = configurationDB(tempDb)

				self.globalDb.createProfile(
					name = k,
					description = profiles[k]["d"],
					databasefile = self.params["mainDatabaseName"],
					oldCfg = profiles[k]["f"])
				if k == defProf:
					self.globalDb.setDefaultProfile(k)
				for p in self.paramOrder:
					if self.params[p] != config_defaults[p]:
						configDb.insert(p, str(self.params[p]))
				tempDb.commit()
				tempDb.closeDB()
				oldfile = os.path.join(self.configPath, profiles[k]["f"])
				os.rename(oldfile, oldfile + "_bck")
				self.logger.info("Old '%s' renamed to '%s'."%(oldfile, oldfile + "_bck"))
			self.globalDb.setProfileOrder(profileOrder)
			self.logger.info([dict(e) for e in self.globalDb.getProfiles()])
			os.rename(self.configProfilesFile, self.configProfilesFile + "_bck")
			self.logger.info("Old '%s' renamed to '%s'."%(self.configProfilesFile, self.configProfilesFile + "_bck"))

	def oldReadProfiles(self):
		"""
		Reads the list of profiles and the related parameters from the profiles.dat file.
		"""
		with open(self.configProfilesFile) as r:
			txtarr = r.readlines()
		txt = "".join(txtarr)
		parsed = ast.literal_eval(txt.replace("\n",""))
		if len(parsed) < 3:
			parsed = parsed + tuple(sorted(parsed[1].keys()))
		return parsed

	def readProfiles(self):
		"""
		Reads the list of profiles and the related parameters from the profiles database.

		Output:
			the name of the default profile, the dictionary with the profiles and the list of ordered profile names
		"""
		allProf = self.globalDb.getProfiles()
		profiles = {}
		for e in allProf:
			profiles[e["name"]] = {
				"n": e["name"],
				"d": e["description"],
				"f": e["oldCfg"],
				"db": e["databasefile"],
			}
		return self.globalDb.getDefaultProfile(), profiles, self.globalDb.getProfileOrder()

	def reInit(self, newShort, newProfile = None):
		"""
		Used when changing profile.
		Reload all the configuration given the new profile name.

		Parameters:
			newShort (str): short name for the new profile to be loaded
			newProfile (dict, optional): the profile file dictionary
		"""
		if newProfile is None:
			try:
				newProfile = self.profiles[newShort]
			except KeyError:
				self.logger.error("Profile not found!")
				return
		self.currentProfileName = newShort
		self.currentDatabase = newProfile["db"]
		self.params = {}
		for k, v in config_defaults.items():
			self.params[k] = v
		self.logger.info("Restarting with profile '%s', database: %s"%(self.currentProfileName, self.currentProfile["db"]))
		self.readConfig()

	def readConfig(self):
		"""
		Read the configuration from the current database.
		Parses the various parameters given their declared type.
		"""
		self.logger.debug("Reading configuration.\n")
		for k, v in config_defaults.items():
			if k == "mainDatabaseName":
				continue
			if type(v) is str and "PBDATA" in v:
				v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
			self.params[k] = v
		tempDb = physbiblioDBCore(self.currentDatabase, self.logger, info = False)
		configDb = configurationDB(tempDb)
		try:
			for k in config_defaults.keys():
				if k == "mainDatabaseName":
					continue
				cont = configDb.getByName(k)
				if len(cont) == 0:
					continue
				v = cont[0]["value"]
				try:
					if config_special[k] == 'float':
						self.params[k] = float(v)
					elif config_special[k] == 'int':
						self.params[k] = int(v)
					elif config_special[k] == 'boolean':
						if v.lower() in ('true','1','yes','on'):
							self.params[k] = True
						elif v.lower() in ('false','0','no','off'):
							self.params[k] = False
						else:
							raise ValueError
					elif config_special[k] == 'list':
						self.params[k] = ast.literal_eval(v.strip())
					else:
						if type(v) is str and "PBDATA" in v:
							v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
						self.params[k] = v
				except Exception:
					self.logger.warning("Failed in reading parameter '%s'."%k, exc_info = True)
					self.params[k] = config_defaults[k]
		except Exception:
			self.logger.error("ERROR: reading config from '%s' failed."%self.currentProfile["db"])
		tempDb.closeDB(info = False)
		self.logger.debug("Configuration loaded.\n")

	def oldReInit(self, newShort, newProfile):
		"""
		Old function used when changing profile.
		Reloads all the configuration from scratch given the new profile name.

		Parameters:
			newShort (str): short name for the new profile to be loaded
			newProfile (dict): the profile file dictionary
		"""
		self.currentProfileName = newShort
		self.params = {}
		for k, v in config_defaults.items():
			self.params[k] = v
		self.currentProfile = newProfile
		self.logger.info("Starting with configuration in '%s'"%self.currentProfile["f"])
		self.configMainFile = os.path.join(self.configPath, self.currentProfile["f"])
		self.params = {}
		self.oldReadConfigFile()

	def oldReadConfigFile(self):
		"""
		Read the configuration from a file, whose name is stored in self.configMainFile.
		Parses the various parameters given their declared type.
		"""
		for k, v in config_defaults.items():
			if type(v) is str and "PBDATA" in v:
				v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
			self.params[k] = v
		try:
			with open(self.configMainFile) as r:
				txt = r.readlines()
			for l in txt:
				k, v = l.replace("\n", "").split(" = ")
				try:
					if config_special[k] == 'float':
						self.params[k] = float(v)
					elif config_special[k] == 'int':
						self.params[k] = int(v)
					elif config_special[k] == 'boolean':
						if v.lower() in ('true','1','yes','on'):
							self.params[k] = True
						elif v.lower() in ('false','0','no','off'):
							self.params[k] = False
						else:
							raise ValueError
					elif config_special[k] == 'list':
						self.params[k] = ast.literal_eval(v.strip())
					else:
						if type(v) is str and "PBDATA" in v:
							v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
						self.params[k] = v
				except Exception:
					self.logger.warning("Failed in reading parameter", exc_info = True)
					self.params[k] = v
		except IOError:
			self.logger.warning("ERROR: config file %s do not exist."%self.configMainFile)
		except Exception:
			self.logger.error("ERROR: reading %s file failed."%self.configMainFile)

pbConfig = ConfigVars()



#!/usr/bin/env python
"""
Test file for the physbiblio.inspireStats module.

This file is part of the physbiblio package.
"""
import sys, traceback, os, datetime
import matplotlib

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch, call
	from StringIO import StringIO
else:
	import unittest
	from unittest.mock import patch, call
	from io import StringIO

try:
	from physbiblio.errors import pBErrorManager
	from physbiblio.setuptests import *
	from physbiblio.inspireStats import pBStats
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

@unittest.skipIf(skipOnlineTests, "Online tests")
class TestInspireStatsMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.inspireStats"""
	@patch('sys.stdout', new_callable=StringIO)
	def assert_in_stdout(self, function, expected_output, mock_stdout):
		"""Catch and if test stdout of the function contains a string"""
		pBErrorManager.tempHandler(sys.stdout, format = '%(message)s')
		function()
		pBErrorManager.rmTempHandler()
		self.assertIn(expected_output, mock_stdout.getvalue())

	def test_paperStats(self):
		"""Test paperStats function downloading real and fake data"""
		#not a valid record (it was cancelled):
		self.assertEqual(pBStats.paperStats("1358853"),
			{'aI': {}, 'citList': [[datetime.date.today()], [0]], 'id': '1358853'})
		self.assert_in_stdout(lambda: pBStats.paperStats("1358853"),
			"Empty response!")

		#needs multiple pages to load all the content
		testGood = pBStats.paperStats("650592")
		self.assertTrue(testGood["id"], "650592")
		self.assertTrue(len(testGood["aI"])>950)
		self.assertEqual(len(testGood["aI"]) + 1, len(testGood["citList"][1]))
		self.assertEqual(len(testGood["citList"][0]), len(testGood["citList"][1]))
		self.assertFalse("fig" in testGood.keys())

		testGood = pBStats.paperStats("1385583", plot = True)
		self.assertTrue(testGood["id"], "1385583")
		self.assertTrue(len(testGood["aI"])>120)
		self.assertEqual(len(testGood["aI"]) + 1, len(testGood["citList"][1]))
		self.assertEqual(len(testGood["citList"][0]), len(testGood["citList"][1]))
		self.assertIsInstance(testGood["fig"], matplotlib.figure.Figure)

	def test_authorStats(self):
		"""Test paperStats function downloading real and fake data"""
		with patch("physbiblio.inspireStats.inspireStatsLoader.paperStats",
				return_value = {'aI': {1653464: {'date': datetime.datetime(2018, 2, 7, 4, 24, 2)}, 1662320: {'date': datetime.datetime(2018, 3, 14, 4, 53, 45)}, 1664547: {'date': datetime.datetime(2018, 3, 29, 5, 8, 58)}, 1663942: {'date': datetime.datetime(2018, 3, 26, 3, 25, 34)}, 1658582: {'date': datetime.datetime(2018, 3, 6, 3, 58, 52)}}, 'id': '1649081', 'citList': [[datetime.datetime(2018, 2, 7, 4, 24, 2), datetime.datetime(2018, 3, 6, 3, 58, 52), datetime.datetime(2018, 3, 14, 4, 53, 45), datetime.datetime(2018, 3, 26, 3, 25, 34), datetime.datetime(2018, 3, 29, 5, 8, 58), datetime.date(2018, 4, 16)], [1, 2, 3, 4, 5, 5]]}) as _mock:
			testGood = pBStats.authorStats("E.M.Zavanin.1", plot = True)
			for i in ['1229039','1345462','1366180','1385583','1387736','1404176','1460832','1519902']:
				self.assertIn(i, testGood["aI"].keys())
			_mock.assert_has_calls([call(i, paperDate = testGood["aI"][i]["date"], verbose = 0) for i in sorted(testGood["aI"].keys())])
			self.assertTrue(testGood.keys(), ['meanLi', 'allLi', 'aI', 'paLi', 'h', 'name'])
			self.assertTrue(testGood["h"], 4)
			self.assertTrue(testGood['name'], 'E.M.Zavanin.1')
			self.assertEqual(len(testGood['paLi'][0]), len(testGood["aI"]))
			self.assertEqual(len(testGood['paLi'][1]), len(testGood["aI"]))
			self.assertEqual(len(testGood['allLi'][0]), 5 * len(testGood["aI"]))
			self.assertEqual(len(testGood['allLi'][1]), 5 * len(testGood["aI"]))
			self.assertEqual(len(testGood['meanLi'][0]), 5 * len(testGood["aI"]))
			self.assertEqual(len(testGood['meanLi'][1]), 5 * len(testGood["aI"]))
			self.assertEqual(len(testGood['figs']), 6)
			for f in testGood["figs"]:
				self.assertIsInstance(f, matplotlib.figure.Figure)

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
import sys
from PySide.QtCore import *
from PySide.QtGui  import *
import traceback
import operator

try:
	from physbiblio.database import *
	from physbiblio.config import pbConfig
	from physbiblio.gui.ErrorManager import pBGUILogger
	from physbiblio.gui.DialogWindows import *
	from physbiblio.gui.CommonClasses import *
	from physbiblio.gui.CatWindows import *
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside: Run script update_resources.sh")

def editExperiment(parent, statusBarObject, editIdExp = None):
	if editIdExp is not None:
		edit = pBDB.exps.getDictByID(editIdExp)
	else:
		edit = None
	newExpWin = editExp(parent, exp = edit)
	newExpWin.exec_()
	data = {}
	if newExpWin.result:
		for k, v in newExpWin.textValues.items():
			s = "%s"%v.text()
			data[k] = s
		if data["name"].strip() != "":
			if "idExp" in data.keys():
				print("[GUI] Updating experiment %s..."%data["idExp"])
				pBDB.exps.update(data, data["idExp"])
			else:
				pBDB.exps.insert(data)
			message = "Experiment saved"
			statusBarObject.setWindowTitle("PhysBiblio*")
			try:
				parent.recreateTable()
			except:
				pass
		else:
			message = "ERROR: empty experiment name"
	else:
		message = "No modifications to experiments"
	try:
		statusBarObject.StatusBarMessage(message)
	except:
		pass

def deleteExperiment(parent, statusBarObject, idExp, name):
	if askYesNo("Do you really want to delete this experiment (ID = '%s', name = '%s')?"%(idExp, name)):
		pBDB.exps.delete(int(idExp))
		statusBarObject.setWindowTitle("PhysBiblio*")
		message = "Experiment deleted"
		try:
			parent.recreateTable()
		except:
			pass
	else:
		message = "Nothing changed"
	try:
		statusBarObject.StatusBarMessage(message)
	except:
		pass

class MyExpTableModel(MyTableModel):
	def __init__(self, parent, exp_list, header, askExps = False, previous = [], *args):
		self.typeClass = "Exps"
		self.dataList = exp_list
		MyTableModel.__init__(self, parent, header, askExps, previous, *args)
		self.prepareSelected()

	def getIdentifier(self, element):
		return element["idExp"]

	def data(self, index, role):
		if not index.isValid():
			return None
		row = index.row()
		column = index.column()
		try:
			value = self.dataList[row][column]
		except IndexError:
			return None

		if role == Qt.CheckStateRole and self.ask and column == 0:
			if self.selectedElements[self.dataList[row][0]] == False:
				return Qt.Unchecked
			else:
				return Qt.Checked
		if role == Qt.EditRole:
			return value
		if role == Qt.DisplayRole:
			return value
		return None

	def setData(self, index, value, role):
		if role == Qt.CheckStateRole and index.column() == 0:
			if value == Qt.Checked:
				self.selectedElements[self.dataList[index.row()][0]] = True
			else:
				self.selectedElements[self.dataList[index.row()][0]] = False

		self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),index, index)
		return True

class ExpWindowList(objListWindow):
	"""create a window for printing the list of experiments"""
	def __init__(self, parent = None, askExps = False, askForBib = None, askForCat = None, previous = []):

		#table dimensions
		self.colcnt = len(pBDB.tableCols["experiments"])
		self.colContents = []
		for j in range(self.colcnt):
			self.colContents.append(pBDB.tableCols["experiments"][j])
		self.askExps = askExps
		self.askForBib = askForBib
		self.askForCat = askForCat
		self.previous = previous

		super(ExpWindowList, self).__init__(parent)
		self.parent = parent
		self.setWindowTitle('List of experiments')

		self.createTable()

	def populateAskExp(self):
		if self.askExps:
			if self.askForBib is not None:
				bibitem = pBDB.bibs.getByBibkey(self.askForBib, saveQuery = False)[0]
				try:
					bibtext = QLabel("Mark categories for the following entry:\n    key:\n%s\n    author(s):\n%s\n    title:\n%s\n"%(self.askForBib, bibitem["author"], bibitem["title"]))
				except:
					bibtext = QLabel("Mark categories for the following entry:\n    key:\n%s\n"%(self.askForBib))
				self.currLayout.addWidget(bibtext)
			elif self.askForCat is not None:
				pass
			self.marked = []
			self.parent.selectedExps = []

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.parent.selectedExps = [idE for idE in self.table_model.selectedElements.keys() if self.table_model.selectedElements[idE] == True]
		self.result	= "Ok"
		self.close()

	def onNewExp(self):
		editExperiment(self.parent, self.parent)
		self.recreateTable()

	def keyPressEvent(self, e):		
		if e.key() == Qt.Key_Escape:
			self.close()

	def changeFilter(self, string):
		self.proxyModel.setFilterRegExp(str(string))

	def createTable(self):
		self.populateAskExp()

		self.exps = pBDB.exps.getAll()

		self.table_model = MyExpTableModel(self, self.exps, pBDB.tableCols["experiments"], askExps = self.askExps, previous = self.previous)
		self.addFilterInput("Filter experiment")
		self.setProxyStuff(1, Qt.AscendingOrder)

		self.finalizeTable()

		self.addExpButton = QPushButton('Add new experiment', self)
		self.addExpButton.clicked.connect(self.onNewExp)
		self.currLayout.addWidget(self.addExpButton)

		if self.askExps:
			self.acceptButton = QPushButton('OK', self)
			self.acceptButton.clicked.connect(self.onOk)
			self.currLayout.addWidget(self.acceptButton)
			
			# cancel button
			self.cancelButton = QPushButton('Cancel', self)
			self.cancelButton.clicked.connect(self.onCancel)
			self.cancelButton.setAutoDefault(True)
			self.currLayout.addWidget(self.cancelButton)

	def triggeredContextMenuEvent(self, row, col, event):
		index = self.tablewidget.model().index(row, col)
		try:
			idExp = str(self.proxyModel.sibling(row, 0, index).data())
			expName = str(self.proxyModel.sibling(row, 1, index).data())
		except AttributeError:
			return
		menu = QMenu()
		titAct = menu.addAction("--Experiment: %s--"%expName).setDisabled(True)
		menu.addSeparator()
		bibAction = menu.addAction("Open list of corresponding entries")
		menu.addSeparator()
		modAction = menu.addAction("Modify")
		delAction = menu.addAction("Delete")
		menu.addSeparator()
		catAction = menu.addAction("Categories")
		action = menu.exec_(event.globalPos())

		if action == bibAction:
			searchDict = {"exps": {"id": [idExp], "operator": "and"}}
			self.parent.reloadMainContent(pBDB.bibs.fetchFromDict(searchDict).lastFetched)
		elif action == modAction:
			editExperiment(self, self.parent, idExp)
		elif action == delAction:
			deleteExperiment(self, self.parent, idExp, expName)
		elif action == catAction:
			previous = [a[0] for a in pBDB.cats.getByExp(idExp)]
			selectCats = catsWindowList(parent = self.parent, askCats = True, askForExp = idExp, expButton = False, previous = previous)
			selectCats.exec_()
			if selectCats.result == "Ok":
				cats = self.parent.selectedCats
				for p in previous:
					if p not in cats:
						pBDB.catExp.delete(p, idExp)
				for c in cats:
					if c not in previous:
						pBDB.catExp.insert(c, idExp)
				self.parent.StatusBarMessage("categories for '%s' successfully inserted"%expName)

	def handleItemEntered(self, index):
		if index.isValid():
			row = index.row()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())
		try:
			expData = pBDB.exps.getByID(idExp)[0]
		except IndexError:
			pBGUILogger.exception("Failed in finding experiment")
			return
		QToolTip.showText(
			QCursor.pos(),
			"{idE}: {exp}\nCorresponding entries: {en}\nAssociated categories: {ca}".format(
				idE = idExp, exp = expData["name"], en = pBDB.bibExp.countByExp(idExp), ca = pBDB.catExp.countByExp(idExp)),
			self.tablewidget.viewport(),
			self.tablewidget.visualRect(index)
		)

	def cellClick(self, index):
		if index.isValid():
			row = index.row()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())

	def cellDoubleClick(self, index):
		if index.isValid():
			row = index.row()
			col = index.column()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())
		if self.colContents[col] == "inspire" or self.colContents[col] == "homepage":
			link = self.proxyModel.sibling(row, col, index).data()
			if link == "":
				return
			if self.colContents[col] == "inspire":
				link = pbConfig.inspireRecord + link
			print("will open '%s' "%link)
			try:
				print("[GUI] opening '%s'..."%link)
				pBGuiView.openLink(link, "link")
			except:
				print("[GUI] opening link '%s' failed!"%link)

class editExp(editObjectWindow):
	"""create a window for editing or creating an experiment"""
	def __init__(self, parent = None, exp = None):
		super(editExp, self).__init__(parent)
		if exp is None:
			self.data = {}
			for k in pBDB.tableCols["experiments"]:
				self.data[k] = ""
		else:
			self.data = exp
		self.createForm()

	def createForm(self):
		self.setWindowTitle('Edit experiment')

		i = 0
		for k in pBDB.tableCols["experiments"]:
			val = self.data[k]
			if k != "idExp" or (k == "idExp" and self.data[k] != ""):
				i += 1
				self.currGrid.addWidget(QLabel(k), i*2-1, 0)
				self.currGrid.addWidget(QLabel("(%s)"%pBDB.descriptions["experiments"][k]), i*2-1, 1)
				self.textValues[k] = QLineEdit(str(val))
				if k == "idExp":
					self.textValues[k].setEnabled(False)
				self.currGrid.addWidget(self.textValues[k], i*2, 0, 1, 2)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currGrid.addWidget(self.acceptButton, i*2+1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currGrid.addWidget(self.cancelButton, i*2+1, 1)

		self.setGeometry(100,100,400, 50*i)
		self.centerWindow()



"""
Module that contains the MyBibTexWriter class (needed to override _entry_to_bibtex).

This file is part of the physbiblio package.
"""
from bibtexparser.bwriter import BibTexWriter

class MyBibTexWriter(BibTexWriter):
	"""This class is used to override _entry_to_bibtex"""
	def __init__(self):
		"""
		Constructor for the MyBibTexWriter class.
		Uses parent class constructor and adds some additional properties.
		"""
		super(MyBibTexWriter, self).__init__()
		#use 13 characters for the field name:
		self._max_field_width = 13
		#order of fields in output
		self.display_order = [
			'author', 'collaboration', 'title', 'booktitle', 'publisher', 'journal', 'volume', 'year', 'pages',
			'russian',
			'archiveprefix', 'primaryclass', 'eprint', 'doi',
			'reportNumber']
		self.bracket_fields = ['title', 'booktitle', 'www', 'note', 'abstract', 'comment', 'article', 'url']
		self.excluded_fields = ["adsnote", "adsurl", "slaccitation"]
		#Necessary to avoid a change of the ordering of the bibtex entries:
		self.order_entries_by = None
		self.comma_first = False

	def _entry_to_bibtex(self, entry):
		"""
		Redefine the function that writes the entries in the bib file.
		Adapted from bibtexparser.bwriter.BibTexWriter._entry_to_bibtex().

		Parameters:
			entry: a dictionary generated by bibtexparser.

		Output:
			the bibtex string
		"""
		bibtex = ''
		# Write BibTeX key
		bibtex += '@' + entry['ENTRYTYPE'].capitalize() + '{' + ( entry['ID'] if entry['ID'] is not None else "" )

		# create display_order of fields for this entry
		# first those keys which are both in self.display_order and in entry.keys
		display_order = [i for i in self.display_order if i in entry]
		# then all the other fields sorted alphabetically
		display_order += [i for i in sorted(entry) if i not in self.display_order and i not in self.excluded_fields]

		# Write field = value lines
		for field in [i for i in display_order if i not in ['ENTRYTYPE', 'ID']]:
			try:
				entry[field] = entry[field].replace("{{{", "{").replace("}}}", "}")
				bibtex += ",\n" + self.indent + "{0:>{1}}".format(field, self._max_field_width) + ' = "' + \
					("{"+entry[field]+"}" if field in self.bracket_fields and (entry[field][0] != "{" or entry[field][-1] != "}") else entry[field]) + '"'
			except TypeError:
				raise TypeError(u"The field %s in entry %s must be a string"
								% (field, entry['ID']))
		bibtex += ",\n}\n" + self.entry_separator
		return bibtex

pbWriter = MyBibTexWriter()



#!/usr/bin/env python
import sys
from PySide.QtCore import *
from PySide.QtGui  import *
import numpy as np
import matplotlib
matplotlib.use('Qt4Agg')
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.patches import Rectangle
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

try:
	from physbiblio.database import *
	from physbiblio.config import pbConfig
	from physbiblio.inspireStats import pBStats
	from physbiblio.gui.DialogWindows import *
	from physbiblio.gui.CommonClasses import *
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside: Run script update_resources.sh")

figTitles = [
"Paper number",
"Papers per year",
"Total citations",
"Citations per year",
"Mean citations",
"Citations for each paper"
]

class authorStatsPlots(QDialog):
	def __init__(self, figs, title = None, parent = None):
		super(authorStatsPlots, self).__init__(parent)
		if title is not None:
			self.setWindowTitle(title)
		self.parent = parent
		layout = QGridLayout(self)
		layout.setSpacing(1)
		self.setLayout(layout)
		self.updatePlots(figs)

		nlines = int(len(figs)/2)
		self.layout().addWidget(QLabel("Click on the lines to have more information:"), nlines + 1, 0)

		self.hIndex = QLabel("Author h index: %d"%self.parent.lastAuthorStats["h"])
		largerFont = QFont("Times", 15, QFont.Bold)
		self.hIndex.setFont(largerFont)
		self.layout().addWidget(self.hIndex, nlines + 1, 1)

		self.textBox = QLineEdit("")
		self.textBox.setReadOnly(True)
		self.layout().addWidget(self.textBox, nlines + 2, 0, 1, 2)
		self.saveButton = QPushButton('Save', self)
		self.saveButton.clicked.connect(self.saveAction)
		self.layout().addWidget(self.saveButton, nlines + 3, 0)

		self.clButton = QPushButton('Close', self)
		self.clButton.clicked.connect(self.close)
		self.clButton.setAutoDefault(True)
		self.layout().addWidget(self.clButton, nlines + 3, 1)

	def saveAction(self):
		savePath = askDirName(self, "Where do you want to save the plots of the stats?")
		if savePath != "":
			self.parent.lastAuthorStats["figs"] = pBStats.plotStats(author = True, save = True, path = savePath)
			infoMessage("Plots saved.")
			self.saveButton.setDisabled(True)

	def pickEvent(self,event):
		ob = event.artist
		ix = -1
		for i, f in enumerate(self.figs):
			if f == ob.figure: ix = i
		if isinstance(ob, Rectangle):
			self.textBox.setText("%s in year %d is: %d"%(figTitles[ix], int(ob._x), int(ob._height)))
		elif isinstance(ob, Line2D):
			xdata = ob.get_xdata()
			ydata = ob.get_ydata()
			ind = event.ind
			if ix == 4:
				formatString = "%s in date %s is %f"
			else:
				formatString = "%s in date %s is %d"
			self.textBox.setText(formatString%(figTitles[ix], np.take(xdata, ind)[0].strftime("%d/%m/%Y"), np.take(ydata, ind)[0]))

	def updatePlots(self, figs):
		i = 0
		while True:
			item = self.layout().takeAt(i)
			if item is None: break
			del item
		if hasattr(self, "canvas"): del self.canvas
		self.figs = figs
		self.canvas = []
		for i,fig in enumerate(figs):
			if fig is not None:
				self.canvas.append(FigureCanvas(fig))
				self.layout().addWidget(self.canvas[-1], int(i/2), i%2)
				self.canvas[-1].mpl_connect("pick_event", self.pickEvent)
				self.canvas[-1].draw()

class paperStatsPlots(QDialog):
	def __init__(self, fig, title = None, parent = None):
		super(paperStatsPlots, self).__init__(parent)
		if title is not None:
			self.setWindowTitle(title)
		self.parent = parent
		layout = QGridLayout(self)
		layout.setSpacing(1)
		self.setLayout(layout)
		self.updatePlots(fig)

		nlines = 1
		self.layout().addWidget(QLabel("Click on the line to have more information:"), nlines + 1, 0)

		self.textBox = QLineEdit("")
		self.textBox.setReadOnly(True)
		self.layout().addWidget(self.textBox, nlines + 2, 0, 1, 2)
		self.saveButton = QPushButton('Save', self)
		self.saveButton.clicked.connect(self.saveAction)
		self.layout().addWidget(self.saveButton, nlines + 3, 0)

		self.clButton = QPushButton('Close', self)
		self.clButton.clicked.connect(self.close)
		self.clButton.setAutoDefault(True)
		self.layout().addWidget(self.clButton, nlines + 3, 1)

	def saveAction(self):
		savePath = askDirName(self, "Where do you want to save the plot of the stats?")
		if savePath != "":
			self.parent.lastPaperStats["fig"] = pBStats.plotStats(paper = True, save = True, path = savePath)
			infoMessage("Plot saved.")
			self.saveButton.setDisabled(True)

	def pickEvent(self,event):
		ob = event.artist
		ix = -1
		if isinstance(ob, Line2D):
			xdata = ob.get_xdata()
			ydata = ob.get_ydata()
			ind = event.ind
			formatString = "%s in date %s is %d"
			self.textBox.setText(formatString%("Citations", np.take(xdata, ind)[0].strftime("%d/%m/%Y"), np.take(ydata, ind)[0]))

	def updatePlots(self, fig):
		i = 0
		while True:
			item = self.layout().takeAt(i)
			if item is None: break
			del item
		if hasattr(self, "canvas"): del self.canvas
		self.fig = fig
		if fig is not None:
			self.canvas = FigureCanvas(fig)
			self.layout().addWidget(self.canvas, 0, 0, 1, 2)
			self.canvas.mpl_connect("pick_event", self.pickEvent)
			self.canvas.draw()



#!/usr/bin/env python
import sys
import os
from PySide.QtCore import *
from PySide.QtGui  import *

try:
	from physbiblio.errors import pBErrorManagerClass, pBLogger
	from physbiblio.view import viewEntry
	from physbiblio.pdf import pBPDF
	from physbiblio.gui.DialogWindows import *
	from physbiblio.database import pBDB, catString
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside.py: Run script update_resources.sh")

class MyLabelRight(QLabel):
	def __init__(self, label):
		super(MyLabelRight, self).__init__(label)
		self.setAlignment(Qt.AlignRight | Qt.AlignVCenter)

class MyLabelCenter(QLabel):
	def __init__(self, label):
		super(MyLabelCenter, self).__init__(label)
		self.setAlignment(Qt.AlignCenter | Qt.AlignVCenter)

class objListWindow(QDialog):
	"""create a window for printing the list of experiments"""
	def __init__(self, parent = None, gridLayout = False):
		"""init using parent class and create common definitions"""
		super(objListWindow, self).__init__(parent)
		self.tableWidth = None
		self.proxyModel = None
		self.gridLayout = gridLayout
		if gridLayout:
			self.currLayout = QGridLayout()
		else:
			self.currLayout = QVBoxLayout()
		self.setLayout(self.currLayout)

	def triggeredContextMenuEvent(self, row, col, event):
		raise NotImplementedError()

	def handleItemEntered(self, index):
		raise NotImplementedError()

	def cellClick(self, index):
		raise NotImplementedError()

	def cellDoubleClick(self, index):
		raise NotImplementedError()

	def changeFilter(self, string):
		self.proxyModel.setFilterRegExp(str(string))

	def addFilterInput(self, placeholderText, gridPos = (1, 0)):
		self.filterInput = QLineEdit("",  self)
		self.filterInput.setPlaceholderText(placeholderText)
		self.filterInput.textChanged.connect(self.changeFilter)

		if self.gridLayout:
			self.currLayout.addWidget(self.filterInput, *gridPos)
		else:
			self.currLayout.addWidget(self.filterInput)
		self.filterInput.setFocus()

	def setProxyStuff(self, sortColumn, sortOrder):
		self.proxyModel = QSortFilterProxyModel(self)
		self.proxyModel.setSourceModel(self.table_model)
		self.proxyModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
		self.proxyModel.setSortCaseSensitivity(Qt.CaseInsensitive)
		self.proxyModel.setFilterKeyColumn(-1)

		self.tablewidget = MyTableView(self)
		self.tablewidget.setModel(self.proxyModel)
		self.tablewidget.setSortingEnabled(True)
		self.tablewidget.setMouseTracking(True)
		self.proxyModel.sort(sortColumn, sortOrder)
		self.currLayout.addWidget(self.tablewidget)

	def finalizeTable(self, gridPos = (1, 0)):
		"""resize the table to fit the contents, connect click and doubleclick functions, add layout"""
		self.tablewidget.resizeColumnsToContents()

		maxh = QDesktopWidget().availableGeometry().height()
		maxw = QDesktopWidget().availableGeometry().width()
		self.setMaximumHeight(maxh)
		self.setMaximumWidth(maxw)

		hwidth = self.tablewidget.horizontalHeader().length()
		swidth = self.tablewidget.style().pixelMetric(QStyle.PM_ScrollBarExtent)
		fwidth = self.tablewidget.frameWidth() * 2

		if self.tableWidth is None:
			if hwidth > maxw - (swidth + fwidth):
				self.tableWidth = maxw - (swidth + fwidth)
			else:
				self.tableWidth = hwidth + swidth + fwidth
		self.tablewidget.setFixedWidth(self.tableWidth)

		self.setMinimumHeight(600)

		self.tablewidget.resizeColumnsToContents()
		self.tablewidget.resizeRowsToContents()

		self.tablewidget.entered.connect(self.handleItemEntered)
		self.tablewidget.clicked.connect(self.cellClick)
		self.tablewidget.doubleClicked.connect(self.cellDoubleClick)

		if self.gridLayout:
			self.currLayout.addWidget(self.tablewidget, *gridPos)
		else:
			self.currLayout.addWidget(self.tablewidget)

	def cleanLayout(self):
		"""delete previous table widget"""
		while True:
			o = self.layout().takeAt(0)
			if o is None: break
			o.widget().deleteLater()

	def recreateTable(self):
		"""delete previous table widget and create a new one"""
		self.cleanLayout()
		self.createTable()

class editObjectWindow(QDialog):
	"""create a window for editing or creating an experiment"""
	def __init__(self, parent = None):
		super(editObjectWindow, self).__init__(parent)
		self.parent = parent
		self.textValues = {}
		self.initUI()

	def keyPressEvent(self, e):		
		if e.key() == Qt.Key_Escape:
			self.onCancel()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.currGrid = QGridLayout()
		self.currGrid.setSpacing(1)
		self.setLayout(self.currGrid)

	def centerWindow(self):
		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class MyThread(QThread):
	def __init__(self,  parent = None):
		QThread.__init__(self, parent)

	def run(self):
		pass

	def setStopFlag(self):
		pass

	finished = Signal()

class WriteStream(MyThread):
	mysignal = Signal(str)
	finished = Signal()
	"""class used to redirect the stdout prints to a window"""
	def __init__(self, queue, parent = None, *args, **kwargs):
		super(WriteStream, self).__init__(parent, *args, **kwargs)
		self.queue = queue
		self.running = True
		self.parent = parent

	def write(self, text):
		"""output is sent to window but also copied to real stdout"""
		self.queue.put(text)

	def run(self):
		while self.running:
			text = self.queue.get()
			self.mysignal.emit(text)
		self.finished.emit()

class MyComboBox(QComboBox):
	def __init__(self, parent, fields, current = None):
		super(MyComboBox, self).__init__(parent)
		for f in fields:
			self.addItem(f)
		if current is not None:
			try:
				self.setCurrentIndex(fields.index(current))
			except ValueError:
				pass

class MyAndOrCombo(MyComboBox):
	def __init__(self, parent, current = None):
		super(MyAndOrCombo, self).__init__(parent, ["AND", "OR"], current = current)

class MyTrueFalseCombo(MyComboBox):
	def __init__(self, parent, current = None):
		super(MyTrueFalseCombo, self).__init__(parent, ["True", "False"], current = current)

class MyTableWidget(QTableWidget):
	def __init__(self, rows, cols, parent):
		super(MyTableWidget, self).__init__(rows, cols, parent)
		self.parent = parent

	def contextMenuEvent(self, event):
		self.parent.triggeredContextMenuEvent(self.rowAt(event.y()), self.columnAt(event.x()), event)

class MyTableView(QTableView):
	def __init__(self, parent):
		super(MyTableView, self).__init__(parent)
		self.parent = parent

	def contextMenuEvent(self, event):
		self.parent.triggeredContextMenuEvent(self.rowAt(event.y()), self.columnAt(event.x()), event)

class MyTableModel(QAbstractTableModel):
	def __init__(self, parent, header, ask = False,  previous = [], *args):
		QAbstractTableModel.__init__(self, parent, *args)
		self.header = header
		self.parentObj = parent
		self.previous = previous
		self.ask = ask

	def changeAsk(self, new = None):
		self.emit(SIGNAL("layoutAboutToBeChanged()"))
		if new is None:
			self.ask = not self.ask
		elif new == True or new == False:
			self.ask = new
		self.emit(SIGNAL("layoutChanged()"))

	def getIdentifier(self, element):
		raise NotImplementedError()

	def prepareSelected(self):
		self.emit(SIGNAL("layoutAboutToBeChanged()"))
		self.selectedElements = {}
		for bib in self.dataList:
			self.selectedElements[self.getIdentifier(bib)] = False
		for prevK in self.previous:
			try:
				self.selectedElements[prevK] = True
			except IndexError:
				pBLogger.exception("[%s] Invalid identifier in previous selection: %s"%(self.typeClass, prevK))
		self.emit(SIGNAL("layoutChanged()"))

	def selectAll(self):
		self.emit(SIGNAL("layoutAboutToBeChanged()"))
		for key in self.selectedElements.keys():
			self.selectedElements[key] = True
		self.emit(SIGNAL("layoutChanged()"))

	def unselectAll(self):
		self.emit(SIGNAL("layoutAboutToBeChanged()"))
		for key in self.selectedElements.keys():
			self.selectedElements[key] = False
		self.emit(SIGNAL("layoutChanged()"))

	def addImage(self, imagePath, height):
		"""create a cell containing an image"""
		return QPixmap(imagePath).scaledToHeight(height)

	def addImages(self, imagePaths, outHeight, height = 48):
		"""create a cell containing multiple images"""
		width = len(imagePaths) * height
		pm = QPixmap(width, height)
		pm.fill(Qt.transparent)
		painter = QPainter(pm)
		for i, img in enumerate(imagePaths):
			painter.drawPixmap(i * height, 0, QPixmap(img))
		painter.end()
		return pm.scaledToHeight(outHeight)

	def rowCount(self, parent = None):
		return len(self.dataList)

	def columnCount(self, parent = None):
		try:
			return len(self.header)
		except IndexError:
			return 0

	def data(self, index, role):
		raise NotImplementedError()

	def flags(self, index):
		if not index.isValid():
			return None
		if index.column() == 0 and self.ask:
			return Qt.ItemIsUserCheckable | Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
		else:
			return Qt.ItemIsEnabled | Qt.ItemIsSelectable

	def headerData(self, col, orientation, role):
		if orientation == Qt.Horizontal and role == Qt.DisplayRole:
			return self.header[col]
		return None

	def setData(self, index, value, role):
		raise NotImplementedError()

	def sort(self, col = 1, order = Qt.AscendingOrder):
		"""sort table by given column number col"""
		self.emit(SIGNAL("layoutAboutToBeChanged()"))
		self.dataList = sorted(self.dataList, key=operator.itemgetter(col) )
		if order == Qt.DescendingOrder:
			self.dataList.reverse()
		self.emit(SIGNAL("layoutChanged()"))

#https://www.hardcoded.net/articles/using_qtreeview_with_qabstractitemmodel
class TreeNode(object):
	def __init__(self, parent, row):
		self.parent = parent
		self.row = row
		self.subnodes = self._getChildren()

	def _getChildren(self):
		raise NotImplementedError()

class TreeModel(QAbstractItemModel):
	def __init__(self):
		QAbstractItemModel.__init__(self)
		self.rootNodes = self._getRootNodes()

	def _getRootNodes(self):
		raise NotImplementedError()

	def index(self, row, column, parent):
		if not parent.isValid():
			return self.createIndex(row, column, self.rootNodes[row])
		parentNode = parent.internalPointer()
		return self.createIndex(row, column, parentNode.subnodes[row])

	def parent(self, index):
		if not index.isValid():
			return QModelIndex()
		node = index.internalPointer()
		if node.parent is None:
			return QModelIndex()
		else:
			return self.createIndex(node.parent.row, 0, node.parent)

	def reset(self):
		self.rootNodes = self._getRootNodes()
		QAbstractItemModel.reset(self)

	def rowCount(self, parent):
		if not parent.isValid():
			return len(self.rootNodes)
		node = parent.internalPointer()
		return len(node.subnodes)

class NamedElement(object): # your internal structure
	def __init__(self, idCat, name, subelements):
		self.idCat = idCat
		self.name = name
		self.text = catString(idCat, pBDB)
		self.subelements = subelements

class NamedNode(TreeNode):
	def __init__(self, ref, parent, row):
		self.ref = ref
		TreeNode.__init__(self, parent, row)

	def _getChildren(self):
		return [NamedNode(elem, self, index)
			for index, elem in enumerate(self.ref.subelements)]

#http://gaganpreet.in/blog/2013/07/04/qtreeview-and-custom-filter-models/
class LeafFilterProxyModel(QSortFilterProxyModel):
	''' Class to override the following behaviour:
			If a parent item doesn't match the filter,
			none of its children will be shown.

		This Model matches items which are descendants
		or ascendants of matching items.
	'''

	def filterAcceptsRow(self, row_num, source_parent):
		''' Overriding the parent function '''

		# Check if the current row matches
		if self.filter_accepts_row_itself(row_num, source_parent):
			return True

		# Traverse up all the way to root and check if any of them match
		if self.filter_accepts_any_parent(source_parent):
			return True

		# Finally, check if any of the children match
		return self.has_accepted_children(row_num, source_parent)

	def filter_accepts_row_itself(self, row_num, parent):
		return super(LeafFilterProxyModel, self).filterAcceptsRow(row_num, parent)

	def filter_accepts_any_parent(self, parent):
		''' Traverse to the root node and check if any of the
			ancestors match the filter
		'''
		while parent.isValid():
			if self.filter_accepts_row_itself(parent.row(), parent.parent()):
				return True
			parent = parent.parent()
		return False

	def has_accepted_children(self, row_num, parent):
		''' Starting from the current node as root, traverse all
			the descendants and test if any of the children match
		'''
		model = self.sourceModel()
		source_index = model.index(row_num, 0, parent)

		children_count =  model.rowCount(source_index)
		for i in range(children_count):
			if self.filterAcceptsRow(i, source_index):
				return True
		return False

class MyDDTableWidget(QTableWidget):
	def __init__(self,  header):
		super(MyDDTableWidget, self).__init__()
		self.setColumnCount(1)
		self.setHorizontalHeaderLabels([header])
		self.setDragEnabled(True)
		self.setAcceptDrops(True)
		self.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.setDragDropOverwriteMode(False)

	# Override this method to get the correct row index for insertion
	def dropMimeData(self, row, col, mimeData, action):
		self.last_drop_row = row
		return True

	def dropEvent(self, event):
		# The QTableWidget from which selected rows will be moved
		sender = event.source()

		# Default dropEvent method fires dropMimeData with appropriate parameters (we're interested in the row index).
		super(MyDDTableWidget, self).dropEvent(event)
		# Now we know where to insert selected row(s)
		dropRow = self.last_drop_row

		selectedRows = sender.getselectedRowsFast()

		# Allocate space for transfer
		for _ in selectedRows:
			self.insertRow(dropRow)

		# if sender == receiver (self), after creating new empty rows selected rows might change their locations
		sel_rows_offsets = [0 if self != sender or srow < dropRow else len(selectedRows) for srow in selectedRows]
		selectedRows = [row + offset for row, offset in zip(selectedRows, sel_rows_offsets)]

		# copy content of selected rows into empty ones
		for i, srow in enumerate(selectedRows):
			for j in range(self.columnCount()):
				item = sender.item(srow, j)
				if item:
					source = QTableWidgetItem(item)
					self.setItem(dropRow + i, j, source)

		# delete selected rows
		for srow in reversed(selectedRows):
			sender.removeRow(srow)

		event.accept()

	def getselectedRowsFast(self):
		selectedRows = []
		for item in self.selectedItems():
			if item.row() not in selectedRows and item.text() != "bibkey":
				selectedRows.append(item.row())
		selectedRows.sort()
		return selectedRows

class MyMenu(QMenu):
	def __init__(self, parent = None):
		super(MyMenu, self).__init__(parent)
		self.possibleActions = []
		self.result = False

	def fillMenu(self):
		for act in self.possibleActions:
			if act is None:
				self.addSeparator()
			elif type(act) is list:
				currmenu = self.addMenu(act[0])
				for a in act[1]:
					currmenu.addAction(a)
			else:
				self.addAction(act)

	def keyPressEvent(self, e):
		if e.key() == Qt.Key_Escape:
			self.close()

class guiViewEntry(viewEntry):
	"""
	Extends viewEntry class to work with QtGui.QDesktopServices
	"""
	def __init__(self):
		"""
		Init the class, storing the name of the external web application
		and the base strings to build some links
		"""
		viewEntry.__init__(self)

	def openLink(self, key, arg = "", fileArg = None):
		if type(key) is list:
			for k in key:
				self.openLink(k, arg, fileArg)
		else:
			if arg is "file":
				url = QUrl.fromLocalFile(fileArg)
			elif arg is "link":
				url = QUrl(key)
			else:
				link = self.getLink(key, arg = arg, fileArg = fileArg)
				url = QUrl(link)
			if QDesktopServices.openUrl(url):
				pBLogger.debug("Opening link '%s' for entry '%s' successful!"%(url.toString(), key))
			else:
				pBLogger.warning("Opening link for '%s' failed!"%key)

pBGuiView = guiViewEntry()

class MyImportedTableModel(MyTableModel):
	def __init__(self, parent, biblist, header, idName = "ID", *args):
		self.typeClass = "imports"
		self.idName = idName
		self.bibsOrder = [k for k in biblist.keys()]
		self.dataList = [biblist[k]['bibpars'] for k in self.bibsOrder]
		self.existList = [biblist[k]['exist'] for k in self.bibsOrder]
		MyTableModel.__init__(self, parent, header, *args)
		self.prepareSelected()

	def getIdentifier(self, element):
		return element[self.idName]

	def data(self, index, role):
		if not index.isValid():
			return None
		row = index.row()
		column = index.column()
		try:
			value = self.dataList[row][self.header[column]]
		except (IndexError, KeyError):
			return None

		if role == Qt.CheckStateRole and column == 0 and self.existList[row] is False:
			if self.selectedElements[self.dataList[row][self.idName]] == False:
				return Qt.Unchecked
			else:
				return Qt.Checked
		if role in [Qt.EditRole, Qt.DisplayRole] and column == 0 and self.existList[row] is True:
			return value + " - already existing"
		if role == Qt.EditRole:
			return value
		if role == Qt.DisplayRole:
			return value
		return None

	def setData(self, index, value, role):
		if role == Qt.CheckStateRole and index.column() == 0:
			if value == Qt.Checked:
				self.selectedElements[self.dataList[index.row()][self.idName]] = True
			else:
				self.selectedElements[self.dataList[index.row()][self.idName]] = False

		self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),index, index)
		return True

	def flags(self, index):
		if not index.isValid():
			return None
		if index.column() == 0 and self.existList[index.row()] is False:
			return Qt.ItemIsUserCheckable | Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
		else:
			return Qt.ItemIsEnabled | Qt.ItemIsSelectable



#!/usr/bin/env python
import sys
from PySide.QtCore import *
from PySide.QtGui  import *
import traceback

if sys.version_info[0] < 3:
	from StringIO import StringIO
else:
	from io import StringIO

try:
	from physbiblio.config import pbConfig
	from physbiblio.gui.CommonClasses import *
	from physbiblio.errors import pBLogger, pBErrorManager
	from physbiblio.database import pBDB
	from physbiblio.webimport.webInterf import physBiblioWeb
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside: Run script update_resources.sh")

def askYesNo(message, title = "Question"):
	reply = QMessageBox.question(None, title, message, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
	if reply == QMessageBox.Yes:
		return True
	if reply == QMessageBox.No:
		return False

def askFileName(parent = None, title = "Filename to use:", filter = ""):
	reply = QFileDialog.getOpenFileName(parent, title, "", filter)
	return reply[0]

def askFileNames(parent = None, title = "Filename to use:", filter = ""):
	reply = QFileDialog.getOpenFileNames(parent, title, "", filter)
	return reply[0]

def askSaveFileName(parent = None, title = "Filename to use:", filter = "", dir = ""):
	reply = QFileDialog.getSaveFileName(parent, title, dir, filter, options = QFileDialog.DontConfirmOverwrite)
	return reply[0]

def askDirName(parent = None, title = "Directory to use:", dir = ""):
	reply = QFileDialog.getExistingDirectory(parent, title, dir, options = QFileDialog.ShowDirsOnly)
	return reply

def askGenericText(message, title, parent = None):
	reply = QInputDialog.getText(parent, title, message)
	return reply

def infoMessage(message, title = "Information"):
	reply = QMessageBox.information(None, title, message)

class configEditColumns(QDialog):
	def __init__(self, parent = None, previous = None):
		super(configEditColumns, self).__init__(parent)
		self.excludeCols = ["crossref", "bibtex", "exp_paper", "lecture", "phd_thesis", "review", "proceeding", "book", "noUpdate"]
		self.moreCols = ["title", "author", "journal", "volume", "pages", "primaryclass", "booktitle", "reportnumber"]
		self.previousSelected = previous if previous is not None else pbConfig.params["bibtexListColumns"]
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result = True
		self.selected = []
		for row in range(self.listSel.rowCount()):
			self.selected.append(self.listSel.item(row, 0).text())
		self.close()

	def initUI(self):
		self.layout = QGridLayout()
		self.setLayout(self.layout)

		self.listAll = MyDDTableWidget("Available columns")
		self.listSel = MyDDTableWidget("Selected columns")
		self.layout.addWidget(QLabel("Drag and drop items to order visible columns"), 0, 0, 1, 2)
		self.layout.addWidget(self.listAll, 1, 0)
		self.layout.addWidget(self.listSel, 1, 1)

		self.allItems = pBDB.descriptions["entries"].keys() + self.moreCols
		self.selItems = self.previousSelected
		i=0
		for col in self.allItems:
			if col not in self.selItems and col not in self.excludeCols:
				item = QTableWidgetItem(col)
				self.listAll.insertRow(i)
				self.listAll.setItem(i, 0, item)
				i += 1
		for i, col in enumerate(self.selItems):
			item = QTableWidgetItem(col)
			self.listSel.insertRow(i)
			self.listSel.setItem(i, 0, item)

		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.layout.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.layout.addWidget(self.cancelButton, 2, 1)

class printText(QDialog):
	"""create a window for printing text of command line output"""
	stopped = Signal()

	def __init__(self, parent = None, title = "", progressBar = True, totStr = None, progrStr = None, noStopButton = False):
		super(printText, self).__init__(parent)
		self.message = None
		if title != "":
			self.title = title
		else:
			self.title = "Redirect print"
		self.setProgressBar = progressBar
		self.noStopButton = noStopButton
		self._want_to_close = False
		self.totString = totStr if totStr is not None else "emptyString"
		self.progressString = progrStr if progrStr is not None else "emptyString"
		self.initUI()

	def closeEvent(self, evnt):
		if self._want_to_close:
			super(printText, self).closeEvent(evnt)
		else:
			evnt.ignore()
			#self.setWindowState(QtCore.Qt.WindowMinimized)

	def initUI(self):
		self.setWindowTitle(self.title)

		grid = QGridLayout()
		grid.setSpacing(1)

		i = 0
		if self.message is not None:
			grid.addWidget(QLabel("%s"%self.message), 0, 0)
			i += 1

		#main text
		self.textEdit = QTextEdit()
		grid.addWidget(self.textEdit)

		if self.setProgressBar:
			self.progressBar = QProgressBar(self)
			grid.addWidget(self.progressBar)

		# cancel button...should learn how to connect it with a thread kill
		if self.noStopButton is not True:
			self.cancelButton = QPushButton('Stop', self)
			self.cancelButton.clicked.connect(self.stopExec)
			self.cancelButton.setAutoDefault(True)
			grid.addWidget(self.cancelButton)
		self.closeButton = QPushButton('Close', self)
		self.closeButton.clicked.connect(self.reject)
		self.closeButton.setDisabled(True)
		grid.addWidget(self.closeButton)

		self.setGeometry(100,100,600, 600)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

	def append_text(self,text):
		if self.setProgressBar:
			if self.totString in text:
				tot = [int(s) for s in text.split() if s.isdigit()][0]
				self.progressBarMax(tot)
			elif self.progressString in text:
				curr = [int(s) for s in text.split() if s.isdigit()][0]
				self.progressBar.setValue(curr)
		self.textEdit.moveCursor(QTextCursor.End)
		self.textEdit.insertPlainText( text )

	def progressBarMin(self, minimum):
		if self.setProgressBar:
			self.progressBar.setMinimum(minimum)

	def progressBarMax(self, maximum):
		if self.setProgressBar:
			self.progressBar.setMaximum(maximum)

	def stopExec(self):
		self.cancelButton.setDisabled(True)
		self.stopped.emit()

	def enableClose(self):
		self._want_to_close = True
		self.closeButton.setEnabled(True)

class searchReplaceDialog(QDialog):
	"""create a window for search and replace"""
	def __init__(self, parent = None):
		super(searchReplaceDialog, self).__init__(parent)
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.setWindowTitle('Search and replace')

		grid = QGridLayout()
		grid.setSpacing(1)

		#search
		grid.addWidget(QLabel("Search: "), 0, 0)
		self.searchEdit = QLineEdit("")
		grid.addWidget(self.searchEdit, 0, 1)

		#replace
		grid.addWidget(QLabel("Replace with: "), 1, 0)
		self.replaceEdit = QLineEdit("")
		grid.addWidget(self.replaceEdit, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class advImportDialog(QDialog):
	"""create a window for the advanced import"""
	def __init__(self, parent = None):
		super(advImportDialog, self).__init__(parent)
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.setWindowTitle('Advanced import')

		grid = QGridLayout()
		grid.setSpacing(1)

		##search
		grid.addWidget(QLabel("Select method: "), 0, 0)
		self.comboMethod = MyComboBox(self,
			["INSPIRE-HEP", "arXiv", "DOI", "ISBN"],
			current = "INSPIRE-HEP")
		grid.addWidget(self.comboMethod, 0, 1)

		grid.addWidget(QLabel("Search string: "), 1, 0)
		self.searchStr = QLineEdit("")
		grid.addWidget(self.searchStr, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)
		self.searchStr.setFocus()

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class advImportSelect(objListWindow):
	"""create a window for the advanced import"""
	def __init__(self, bibs = [], parent = None):
		self.bibs = bibs
		self.parent = parent
		super(advImportSelect, self).__init__(parent, gridLayout = True)
		self.checkBoxes = []
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.selected = self.table_model.selectedElements
		self.result	= True
		self.close()

	def keyPressEvent(self, e):
		if e.key() == Qt.Key_Escape:
			self.result	= False
			self.close()

	def changeFilter(self, string):
		self.proxyModel.setFilterRegExp(str(string))

	def initUI(self):
		self.setWindowTitle('Advanced import - results')

		self.currLayout.setSpacing(1)

		self.currLayout.addWidget(QLabel("This is the list of elements found.\nSelect the ones that you want to import:"))

		headers = ["ID", "title", "author", "eprint", "doi"]
		for k in self.bibs.keys():
			try:
				self.bibs[k]['bibpars']["eprint"] = self.bibs[k]['bibpars']["arxiv"]
			except KeyError:
				pass
			try:
				self.bibs[k]['bibpars']["author"] = self.bibs[k]['bibpars']["authors"]
			except KeyError:
				pass
			for f in headers:
				try:
					self.bibs[k]['bibpars'][f] = self.bibs[k]['bibpars'][f].replace("\n", " ")
				except KeyError:
					pass
		self.table_model = MyImportedTableModel(self, self.bibs, headers)
		self.addFilterInput("Filter entries", gridPos = (1, 0))
		self.setProxyStuff(0, Qt.AscendingOrder)

		self.finalizeTable(gridPos = (2, 0, 1, 2))

		i = 3
		self.askCats = QCheckBox("Ask categories at the end?", self)
		self.askCats.toggle()
		self.currLayout.addWidget(self.askCats, i, 0)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currLayout.addWidget(self.acceptButton, i + 1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currLayout.addWidget(self.cancelButton, i + 2, 0)

	def triggeredContextMenuEvent(self, row, col, event):
		pass

	def handleItemEntered(self, index):
		pass

	def cellClick(self, index):
		pass

	def cellDoubleClick(self, index):
		pass

class arxivDailyDialog(QDialog):
	"""create a window for the advanced import"""
	def __init__(self, parent = None):
		super(arxivDailyDialog, self).__init__(parent)
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def updateCat(self, category):
		self.comboSub.clear()
		self.comboSub.addItems(["--"] + physBiblioWeb.webSearch["arxiv"].categories[category])

	def initUI(self):
		self.setWindowTitle('Browse arxiv daily')

		self.grid = QGridLayout()
		self.grid.setSpacing(1)

		##search
		self.grid.addWidget(QLabel("Select category: "), 0, 0)
		self.comboCat = MyComboBox(self,
			[""] + sorted(physBiblioWeb.webSearch["arxiv"].categories.keys()))
		self.comboCat.currentIndexChanged[str].connect(self.updateCat)
		self.grid.addWidget(self.comboCat, 0, 1)

		self.grid.addWidget(QLabel("Subcategory: "), 1, 0)
		self.comboSub = MyComboBox(self,
			[""])
		self.grid.addWidget(self.comboSub, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(self.grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class dailyArxivSelect(advImportSelect):
	"""create a window for the advanced import"""
	def initUI(self):
		self.setWindowTitle('ArXiv daily listing - results')

		self.currLayout.setSpacing(1)

		self.currLayout.addWidget(QLabel("This is the list of elements found.\nSelect the ones that you want to import:"))

		headers = ["eprint", "type", "title", "author", "primaryclass"]
		self.table_model = MyImportedTableModel(self, self.bibs, headers, idName = "eprint")
		self.addFilterInput("Filter entries", gridPos = (1, 0))
		self.setProxyStuff(0, Qt.AscendingOrder)

		self.finalizeTable(gridPos = (2, 0, 1, 2))

		i = 3
		self.askCats = QCheckBox("Ask categories at the end?", self)
		self.askCats.toggle()
		self.currLayout.addWidget(self.askCats, i, 0)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currLayout.addWidget(self.acceptButton, i + 1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currLayout.addWidget(self.cancelButton, i + 2, 0)

		self.abstractArea = QTextEdit('Abastract', self)
		self.currLayout.addWidget(self.abstractArea, i + 3, 0, 4, 2)

	def cellClick(self, index):
		row = index.row()
		try:
			eprint = str(self.proxyModel.sibling(row, 0, index).data())
		except AttributeError:
			return
		if self.abstractFormulas is not None:
			a = self.abstractFormulas(self.parent, self.bibs[eprint]["bibpars"]["abstract"], customEditor = self.abstractArea, statusMessages = False)
			a.doText()



"""
Module for retrieving bibtex information from a list of given web resources (see `__all__`).

This module is part of the physbiblio package.
"""
import physbiblio
__version__ = physbiblio.__version__

__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__all__ = ["arxiv", "doi", "inspire", "inspireoai", "isbn", "webInterf"]



"""
Module that deals with importing info from the INSPIRE-HEP API.

This file is part of the physbiblio package.
"""
import traceback
try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	from physbiblio.webimport.webInterf import *
	from physbiblio.parse_accents import *
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
	print(traceback.format_exc())
	raise

class webSearch(webInterf):
	"""Subclass of webInterf that can connect to INSPIRE-HEP to perform searches"""
	def __init__(self):
		"""
		Initializes the class variables using the webInterf constructor.

		Define additional specific parameters for the INSPIRE-HEP API.
		"""
		webInterf.__init__(self)
		self.name = "inspire"
		self.description = "INSPIRE fetcher"
		self.url = pbConfig.inspireSearchBase
		self.urlRecord = pbConfig.inspireRecord
		self.urlArgs = {
			#"action_search": "Search",
			"sf": "year",
			"so": "a",
			"rg": "250",
			"sc": "0",
			"eb": "B",
			"of": "hx"#for bibtex format ---- hb for standard format, for retrieving inspireid
			}
		
	def retrieveUrlFirst(self, string):
		"""
		Retrieves the first result from the content of the given web page.

		Parameters:
			string: the search string

		Output:
			returns the bibtex string obtained from the API
		"""
		self.urlArgs["p"] = string.replace(" ", "+")
		url = self.createUrl()
		pBLogger.info("Search '%s' -> %s"%(string, url))
		text = self.textFromUrl(url)
		try:
			i1 = text.find("<pre>")
			i2 = text.find("</pre>")
			if i1 > 0 and i2 > 0:
				bibtex = text[i1 + 5 : i2]
			else:
				bibtex = ""
			return parse_accents_str(bibtex)
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""
		
	def retrieveUrlAll(self, string):
		"""
		Retrieves all the result from the content of the given web page.

		Parameters:
			string: the search string

		Output:
			returns the bibtex string obtained from the API
		"""
		self.urlArgs["p"] = string.replace(" ", "+")
		url = self.createUrl()
		pBLogger.info("Search '%s' -> %s"%(string, url))
		text = self.textFromUrl(url)
		try:
			i1 = text.find("<pre>")
			i2 = text.rfind("</pre>")
			if i1 > 0 and i2 > 0:
				bibtex = text[i1 + 5 : i2]
			else:
				bibtex = ""
			return parse_accents_str(bibtex.replace("<pre>", "").replace("</pre>", ""))
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""
	
	def retrieveInspireID(self, string, number = None):
		"""
		Read the fetched content for a given entry to obtain its INSPIRE-HEP ID

		Parameters:
			string: the search string
			number (optional): the integer corresponding to the desired entry in the list, if more than one is present
		"""
		i = 0
		self.urlArgs["p"] = string.replace(" ", "+")
		self.urlArgs["of"] = "hb" #do not ask bibtex, but standard
		url = self.createUrl()
		self.urlArgs["of"] = "hx" #restore
		pBLogger.info("Search ID of %s -> %s"%(string, url))
		text = self.textFromUrl(url)
		if text is None:
			pBLogger.warning("An error occurred. Empty text obtained")
			return ""
		try:
			searchID = re.compile('titlelink(.*)?(http|https)://inspirehep.net/record/([0-9]*)?">')
			inspireID = ""
			for q in searchID.finditer(text):
				if len(q.group()) > 0:
					if number is None or i == number:
						inspireID = q.group(3)
						break
					else:
						i += 1
			pBLogger.info("Found: %s"%inspireID)
			return inspireID
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""



"""
Tests for the modules in the physbiblio package.

This file is part of the physbiblio package.
"""
import physbiblio
__version__ = physbiblio.__version__

__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__all__ = ["test_bibtexwriter", "test_database", "test_errors", "test_export", "test_pdf", "test_view"]




import sys
import logging
import traceback
from PySide.QtCore import *
from PySide.QtGui  import *

if sys.version_info[0] < 3:
	from StringIO import StringIO
else:
	from io import StringIO

try:
	from physbiblio.errors import pBErrorManagerClass, pBLogger
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")

class ErrorStream(StringIO):
	def __init__(self, *args, **kwargs):
		StringIO.__init__(self, *args, **kwargs)
		self.priority = 1

	def setPriority(self, prio):
		self.priority = prio

	def write(self, text):
		if text.strip() == "":
			return
		text = text.replace('\n', '<br>')
		if self.priority == 0:
			QMessageBox.information(None, "Information", text)
		elif self.priority > 1:
			QMessageBox.critical(None, "Error", text)
		else:
			QMessageBox.warning(None, "Warning", text)
		self.priority = 1

class pBErrorManagerClassGui(pBErrorManagerClass):
	"""
	Extends the error manager class to show gui messages
	"""
	def __init__(self):
		"""
		Init the class, storing the name of the external web application
		and the base strings to build some links
		"""
		pBErrorManagerClass.__init__(self, "physbiblioguilog")
		self.guiStream = ErrorStream()
		self.tempHandler(self.guiStream, level = logging.DEBUG, format = '%(message)s')

	def loggerPriority(self, prio):
		self.guiStream.priority = prio
		return self.logger

pBGUIErrorManager = pBErrorManagerClassGui()
pBGUILogger = pBGUIErrorManager.logger



"""
Module that manages the actions on the database (and few more).

This file is part of the physbiblio package.
"""
import sqlite3
from sqlite3 import OperationalError, ProgrammingError, DatabaseError, InterfaceError
import os
import ast

try:
	import physbiblio.tablesDef
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
	print(traceback.format_exc())
	raise

encoding_default = 'iso-8859-15'

class physbiblioDBCore():
	"""
	Contains most of the basic functions on the database.
	Will be subclassed to do everything else.
	"""
	def __init__(self, dbname, logger, noOpen = False, info = True):
		"""
		Initialize database class (column names, descriptions) and opens the database.

		Parameters:
			dbname: the name of the database to be opened
			logger: the logger used for messages
			noOpen (boolean, default False): open the database or not
			info (boolean, default True): show some output when opening DB
		"""
		#structure of the tables
		self.tableFields = physbiblio.tablesDef.tableFields
		self.descriptions = physbiblio.tablesDef.fieldsDescriptions
		#names of the columns
		self.tableCols = {}
		for q in self.tableFields.keys():
			self.tableCols[q] = [ a[0] for a in self.tableFields[q] ]

		self.dbChanged = False
		self.conn = None
		self.curs = None
		self.dbname = dbname
		self.logger = logger
		db_is_new = not os.path.exists(self.dbname)

		if not noOpen:
			self.openDB(info = info)
			if db_is_new or self.checkExistingTables():
				self.logger.info("-------New database or missing tables. Creating them!\n\n")
				self.createTables()

		self.lastFetched = None
		self.catsHier = None

		self.loadSubClasses()

	def openDB(self, info = True):
		"""
		Open the database and creates the self.conn (connection) and self.curs (cursor) objects.

		Parameters:
			info (boolean, default True): show some output when opening DB

		Output:
			True
		"""
		if info:
			self.logger.info("Opening database: %s"%self.dbname)
		else:
			self.logger.debug("Opening database: %s"%self.dbname)
		self.conn = sqlite3.connect(self.dbname, check_same_thread=False)
		self.conn.row_factory = sqlite3.Row
		self.curs = self.conn.cursor()
		self.loadSubClasses()
		return True

	def reOpenDB(self):
		"""
		Not defined at this stage. Present in subclass physbiblioDB
		"""
		pass

	def closeDB(self, info = True):
		"""
		Close the database.

		Parameters:
			info (boolean, default True): show some output when opening DB
		"""
		if info:
			self.logger.info("Closing database...")
		else:
			self.logger.debug("Closing database...")
		self.conn.close()
		return True

	def checkUncommitted(self):
		"""
		Check if there are uncommitted changes.

		Output:
			True/False
		"""
		#return self.conn.in_transaction() #works only with sqlite > 3.2...
		return self.dbChanged

	def commit(self, verbose = True):
		"""
		Commit the changes.

		Parameters:
			verbose (boolean, default True): show some output when opening DB

		Output:
			True if successfull, False if an exception occurred
		"""
		try:
			self.conn.commit()
			self.dbChanged = False
			if verbose:
				self.logger.info("Database saved.")
			return True
		except Exception:
			self.logger.exception("Impossible to commit!")
			return False

	def undo(self, verbose = True):
		"""
		Undo the uncommitted changes and roll back to the last commit.

		Parameters:
			verbose (boolean, default True): show some output when opening DB

		Output:
			True if successfull, False if an exception occurred
		"""
		try:
			self.conn.rollback()
			self.dbChanged = False
			if verbose:
				self.logger.info("Rolled back to last commit.")
			return True
		except Exception:
			self.logger.exception("Impossible to rollback!")
			return False
		
	def connExec(self, query, data = None):
		"""
		Execute connection.

		Parameters:
			query (string): the query to be executed
			data (dictionary or list): the values of the parameters in the query

		Output:
			True if successfull, False if an exception occurred
		"""
		try:
			if data:
				self.conn.execute(query,data)
			else:
				self.conn.execute(query)
		except (OperationalError, ProgrammingError, DatabaseError, InterfaceError) as err:
			self.logger.exception('Connection error: %s\nquery: %s'%(err, query))
			return False
		except IntegrityError as err:
			self.logger.exception('Cannot insert/update: ID exists!\n%s\nquery: %s'%(err, query))
			return False
		else:
			self.dbChanged = True
			return True

	def cursExec(self, query, data = None):
		"""
		Execute cursor.

		Parameters:
			query (string): the query to be executed
			data (dictionary or list): the values of the parameters in the query

		Output:
			True if successfull, False if an exception occurred
		"""
		try:
			if data:
				self.curs.execute(query,data)
			else:
				self.curs.execute(query)
		except Exception as err:
			self.logger.exception('Cursor error: %s\nThe query was: "%s"\n and the parameters: %s'%(err, query, data))
			return False
		else:
			return True

	def cursor(self):
		"""
		Function wrapper that returns the default cursor
		"""
		return self.curs

	def loadSubClasses(self):
		"""
		Not defined at this stage. Present in subclass physbiblioDB
		"""
		pass

	def checkExistingTables(self, wantedTables = ["categories", "entries", "entryCats", "entryExps", "expCats", "experiments", "settings"]):
		"""
		Check that all the required tables are present in the database
		"""
		self.cursExec("SELECT name FROM sqlite_master WHERE type='table';")
		tables = [name[0] for name in self.curs]
		return not all(t in tables for t in wantedTables)

	def createTables(self, fieldsDict = None):
		"""
		Create tables for the database (and insert the default categories), if it is missing.

		Parameters:
			fieldsDict (default None): the structure of the tables (see physbiblio.tablesDef)
		"""
		if fieldsDict is None:
			fieldsDict = self.tableFields
		self.cursExec("SELECT name FROM sqlite_master WHERE type='table';")
		existingTables = [name[0] for name in self.curs]
		for q in fieldsDict.keys():
			if q in existingTables:
				continue
			command = "CREATE TABLE %s (\n"%q
			first = True
			for el in fieldsDict[q]:
				if first:
					first = False
				else:
					command += ",\n"
				command += " ".join(el)
			command += ");"
			self.logger.info(command + "\n")
			if not self.connExec(command):
				self.logger.critical("Create %s failed"%q)
		self.cursExec("select * from categories where idCat = 0 or idCat = 1\n")
		if len(self.curs.fetchall()) < 2:
			command="""
			INSERT into categories (idCat, name, description, parentCat, ord)
				values (0,"Main","This is the main category. All the other ones are subcategories of this one",0,0),
				(1,"Tags","Use this category to store tags (such as: ongoing projects, temporary cats,...)",0,0)
				"""
			self.logger.info(command + "\n")
			if not self.connExec(command):
				self.logger.error("Insert main categories failed")
		self.commit()

class physbiblioDBSub():
	"""
	Uses physbiblioDB instance 'self.mainDB = parent' to act on the database.
	All the subcategories of physbiblioDB are defined starting from this one.
	"""
	def __init__(self, parent):
		"""
		Initialize DB class, connecting to the main physbiblioDB instance (parent).
		"""
		self.mainDB = parent
		#structure of the tables
		self.tableFields = self.mainDB.tableFields
		#names of the columns
		self.tableCols = {}
		for q in self.tableFields.keys():
			self.tableCols[q] = [ a[0] for a in self.tableFields[q] ]

		self.conn = self.mainDB.conn
		self.curs = self.mainDB.curs
		self.dbname = self.mainDB.dbname

		self.lastFetched = None
		self.catsHier = None

	def literal_eval(self, string):
		"""
		Wrapper for ast.literal_eval

		Parameters:
			string: the string to evaluate

		Output:
			a string or None
		"""
		try:
			if "[" in string and "]" in string:
				return ast.literal_eval(string.strip())
			elif "," in string:
				return ast.literal_eval("[%s]"%string.strip())
			else:
				return string.strip()
		except SyntaxError:
			self.mainDB.logger.warning("Error in literal_eval with string '%s'"%string)
			return None

	def closeDB(self):
		"""
		Close the database (using physbiblioDB.close)
		"""
		self.mainDB.closeDB()

	def commit(self):
		"""
		Commit the changes (using physbiblioDB.commit)
		"""
		self.mainDB.commit()

	def connExec(self,query,data=None):
		"""
		Execute connection (see physbiblioDB.connExec)
		"""
		return self.mainDB.connExec(query, data = data)

	def cursExec(self, query, data = None):
		"""
		Execute cursor (see physbiblioDB.cursExec)
		"""
		return self.mainDB.cursExec(query, data = data)

	def cursor(self):
		"""
		Return the cursor
		"""
		return self.mainDB.cursor()



#!/usr/bin/env python
"""
Test file for the physbiblio.pdf module.

This file is part of the physbiblio package.
"""
import sys, traceback, os
import shutil

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch
else:
	import unittest
	from unittest.mock import patch

try:
	from physbiblio.setuptests import *
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB, physbiblioDB
	from physbiblio.pdf import pBPDF
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

pBPDF.pdfDir = os.path.join(pbConfig.dataPath, "testpdf_%s"%today_ymd)

@unittest.skipIf(skipLongTests, "Long tests")
class TestPdfMethods(unittest.TestCase):
	"""
	Test the methods and functions in the pdf module
	"""
	def test_fnames(self):
		"""Test names of folders and directories"""
		self.assertEqual(pBPDF.badFName(r'a\b/c:d*e?f"g<h>i|' + "j'"),
			"a_b_c_d_e_f_g_h_i_j_")
		self.assertEqual(pBPDF.getFileDir(r'a\b/c:d*e?f"g<h>i|' + "j'"),
			os.path.join(pBPDF.pdfDir, "a_b_c_d_e_f_g_h_i_j_"))
		testPaperFolder = pBPDF.getFileDir("abc.def")
		with patch('physbiblio.database.entries.getField', return_value="12345678") as _mock:
			self.assertEqual(pBPDF.getFilePath("abc.def", "arxiv"),
				os.path.join(testPaperFolder, "12345678.pdf"))

	def test_manageFiles(self):
		"""Test creation, copy and deletion of files and folders"""
		emptyPdfName = os.path.join(pBPDF.pdfDir, "tests_%s.pdf"%today_ymd)
		pBPDF.createFolder("abc.def")
		self.assertTrue(os.path.exists(pBPDF.getFileDir("abc.def")))
		pBPDF.renameFolder("abc.def", "abc.fed")
		self.assertFalse(os.path.exists(pBPDF.getFileDir("abc.def")))
		self.assertTrue(os.path.exists(pBPDF.getFileDir("abc.fed")))
		open(emptyPdfName, 'a').close()
		with patch('physbiblio.database.entries.getField', return_value="12345678") as _mock:
			pBPDF.copyNewFile("abc.fed", emptyPdfName, "arxiv")
			pBPDF.copyNewFile("abc.fed", emptyPdfName,
				customName = "empty.pdf")
			self.assertTrue(os.path.exists(pBPDF.getFilePath("abc.fed", "arxiv")))
			os.remove(emptyPdfName)
			self.assertFalse(os.path.exists(emptyPdfName))
			pBPDF.copyToDir(pBPDF.pdfDir, "abc.fed", "arxiv")
			self.assertTrue(os.path.exists(os.path.join(pBPDF.pdfDir, "12345678.pdf")))
			self.assertTrue(pBPDF.removeFile("abc.fed", "arxiv"))
			self.assertFalse(pBPDF.removeFile("abc.fed", "arxiv"))
			os.remove(os.path.join(pBPDF.pdfDir, "12345678.pdf"))
			shutil.rmtree(pBPDF.getFileDir("abc.fed"))

	@unittest.skipIf(skipOnlineTests, "Long tests")
	def test_download(self):
		"""Test downloadArxiv"""
		with patch('physbiblio.database.entries.getField', return_value="1806.11344") as _mock:
			self.assertTrue(pBPDF.downloadArxiv("abc.def"))
			self.assertTrue(pBPDF.checkFile("abc.def", "arxiv"))
			self.assertEqual(pBPDF.getExisting("abc.def", fullPath = False),
				["1806.11344.pdf"])
			self.assertEqual(pBPDF.getExisting("abc.def", fullPath = True),
				[os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344.pdf")])
			open(os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344"), "w").close()
			self.assertEqual(pBPDF.getExisting("abc.def", fullPath = False),
				["1806.11344.pdf"])
			self.assertTrue(pBPDF.removeFile("abc.def", "arxiv"))
			self.assertTrue(pBPDF.removeFile("abc.def", "file", os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344")))
			self.assertFalse(pBPDF.checkFile("abc.def", "arxiv"))
		with patch('physbiblio.database.entries.getField',
				side_effect = ["1801.15000", "1801.15000", "", "", None, None]) as _mock:
			self.assertFalse(pBPDF.downloadArxiv("abc.def"))
			self.assertFalse(pBPDF.downloadArxiv("abc.def"))
			self.assertFalse(pBPDF.downloadArxiv("abc.def"))
		shutil.rmtree(pBPDF.getFileDir("abc.def"))

	def test_removeSpare(self):
		"""Test finding spare folders"""
		with patch('physbiblio.database.entries.fetchCursor',
				return_value=[{"bibkey":"abc"}, {"bibkey":"def"}]) as _mock:
			for q in ["abc", "def", "ghi"]:
				pBPDF.createFolder(q)
				self.assertTrue(os.path.exists(pBPDF.getFileDir(q)))
			pBPDF.removeSparePDFFolders()
			for q in ["abc", "def"]:
				self.assertTrue(os.path.exists(pBPDF.getFileDir(q)))
			self.assertFalse(os.path.exists(pBPDF.getFileDir("ghi")))
		shutil.rmtree(pBPDF.pdfDir)

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
"""
Script for installing, uploading, doing tests and so on.

This file is part of the physbiblio package.
"""

from setuptools import setup

import physbiblio

def readme():
    with open('README.md') as f:
        return f.read()

setup(name='physbiblio',
		version=physbiblio.__version__,
		description='A bibliography manager in Python (using Sqlite and PySide)',
		long_description_content_type="text/markdown",
		long_description=readme(),
		author='Stefano Gariazzo',
		author_email='stefano.gariazzo@gmail.com',
		url='https://github.com/steog88/physBiblio',
		license="GPL-3.0",
		keywords="bibliography hep-ph high-energy-physics bibtex",

		packages=['physbiblio', 'physbiblio.gui', 'physbiblio.webimport',
			'physbiblio.tests', 'physbiblio.webimport.tests'],
		scripts=['PhysBiblio'],
		package_data={
			'': ['*.sh', '*.md', '*.png'],
			'physbiblio.gui': ['images/*.png'],
		},
		install_requires=[
			'appdirs',
			'argparse',
			'bibtexparser(>=1.0.1)',
			'dictdiffer',
			'feedparser',
			'matplotlib',
			'outdated',
			'pylatexenc',
			'pymarc',
			'pyoai',
			'requests',
			'six',
			'mock;python_version<"3"',
			'unittest2;python_version<"3"',
			],
		provides=['physbiblio'],
		data_files = [("physbiblio", ["LICENSE", "CHANGELOG", 'physbiblio/gui/images/icon.png'])],
		test_loader = "physbiblio.testLoader:MyScanningLoader",
	)



#!/usr/bin/env python
"""
Test file for the physbiblio.webimport subpackage.

This file is part of the physbiblio package.
"""
import sys, datetime, traceback, os

if sys.version_info[0] < 3:
	import unittest2 as unittest
else:
	import unittest

try:
	from physbiblio.setuptests import *
	from physbiblio.webimport.webInterf import physBiblioWeb
	from physbiblio.webimport.inspireoai import get_journal_ref_xml
	from physbiblio.config import pbConfig
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

@unittest.skipIf(skipOnlineTests, "Online tests")
class TestWebImportMethods(unittest.TestCase):
	"""
	Test the functions that import entries from the web.
	Should not fail if everything works fine.
	"""

	def test_methods_success(self):
		"""Test webimport with known results"""
		print(physBiblioWeb.webSearch.keys())
		tests = {
			"arxiv": ["1507.08204", """@Article{1507.08204,
         title = "{Light sterile neutrinos}",
          year = "2015",
 archiveprefix = "arXiv",
  primaryclass = "hep-ph",
           doi = "10.1088/0954-3899/43/3/033001",
      abstract = "{The theory and phenomenology of light sterile neutrinos at the eV mass scale is reviewed. The reactor, Gallium and LSND anomalies are briefly described and interpreted as indications of the existence of short-baseline oscillations which require the existence of light sterile neutrinos. The global fits of short-baseline oscillation data in 3+1 and 3+2 schemes are discussed, together with the implications for beta-decay and neutrinoless double-beta decay. The cosmological effects of light sterile neutrinos are briefly reviewed and the implications of existing cosmological data are discussed. The review concludes with a summary of future perspectives.}",
         arxiv = "1507.08204",
       authors = "S. Gariazzo and C. Giunti and M. Laveder and Y. F. Li and E. M. Zavanin",
}"""],
			"doi": ["10.1088/0954-3899/43/3/033001", """@article{Gariazzo_2015,
	doi = {10.1088/0954-3899/43/3/033001},
	url = {""" + pbConfig.doiUrl + """10.1088%2F0954-3899%2F43%2F3%2F033001},
	year = 2015,
	month = {mar},
	publisher = {{IOP} Publishing},
	volume = {43},
	number = {3},
	pages = {033001},
	author = {S Gariazzo and C Giunti and M Laveder and Y F Li and E M Zavanin},
	title = {Light sterile neutrinos},
	journal = {Journal of Physics G: Nuclear and Particle Physics}
}"""],
			"inspire": ["Gariazzo:2015rra", """@article{Gariazzo:2015rra,
      author         = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y. F.
                        and Zavanin, E. M.",
      title          = "{Light sterile neutrinos}",
      journal        = "J. Phys.",
      volume         = "G43",
      year           = "2016",
      pages          = "033001",
      doi            = "10.1088/0954-3899/43/3/033001",
      eprint         = "1507.08204",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:1507.08204;%%"
}"""],
			"inspireoai": ["1385583",
				{'doi': '10.1088/0954-3899/43/3/033001', 'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra', 'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.', 'volume': 'G43', 'year': '2016', 'pages': '033001', 'firstdate': '2015-07-29', 'pubdate': '2016-01-13', 'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.', 'collaboration': None, 'primaryclass': 'hep-ph', 'archiveprefix': 'arXiv', 'eprint': '1507.08204', 'title': 'Light sterile neutrinos', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'bibtex': '@Article{Gariazzo:2015rra,\n        author = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.",\n         title = "{Light sterile neutrinos}",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n archiveprefix = "arXiv",\n  primaryclass = "hep-ph",\n        eprint = "1507.08204",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n', 'id': '1385583', 'link': '%s10.1088/0954-3899/43/3/033001'%pbConfig.doiUrl, 'reportnumber': None}],
			"isbn": ["9780198508717", """@book{9780198508717,
  Author = {Carlo Giunti, Chung W. Kim},
  Title = {Fundamentals of Neutrino Physics and Astrophysics},
  Publisher = {OXFORD UNIV PR},
  Year = {2007},
  Date = {2007-05-11},
  PageTotal = {710 Seiten},
  EAN = {9780198508717},
  ISBN = {0198508719},
  URL = {https://www.ebook.de/de/product/7616244/carlo_giunti_chung_w_kim_fundamentals_of_neutrino_physics_and_astrophysics.html}
}"""],
			}
		for method, strings in tests.items():
			print(method)
			if method == "inspireoai":
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveOAIData(strings[0]), strings[1])
			elif method == "arxiv":
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0], searchType = "id").strip(), strings[1].strip())
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0], searchType = "id").strip(), strings[1].strip())
			else:
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0]).strip(), strings[1].strip())
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(), strings[1].strip())
		self.assertEqual(physBiblioWeb.webSearch["inspire"].retrieveInspireID(tests["inspire"][0]), u'1385583')

	def test_methods_insuccess(self):
		"""Test webimport using missing and/or invalid identifiers"""
		print(physBiblioWeb.webSearch.keys())
		tests = {
			"arxiv": ["1801.15000", ""],
			"doi": ["10.1088/9999-3899/43/a/033001", ""],
			"inspire": ["Gariazzo:2014rra", ""],
			"isbn": ["978019850871a", ""],
			}
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("1110620"),
			{'doi': None, 'arxiv': None, 'bibkey': None, 'ads': None, 'journal': None, 'volume': None, 'year': None, 'pages': None, 'firstdate': None, 'pubdate': None, 'author': ' and Tauber, Jan', 'collaboration': 'Planck', 'title': 'Anisotropies of the Cosmic Background Radiation Field', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'bibtex': '@Article{,\n        author = " and Tauber, Jan",\n collaboration = "Planck",\n         title = "{Anisotropies of the Cosmic Background Radiation Field}",\n}\n\n', 'id': '1110620', 'archiveprefix': None, 'eprint': None, 'link': None, 'primaryclass': None, 'reportnumber': None,})
		self.assertFalse(physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("9999999"))
		for method, strings in tests.items():
			print(method)
			res=physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0])
			print(res)
			self.assertEqual(res.strip(), strings[1].strip())
			self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(), strings[1].strip())
		self.assertEqual(physBiblioWeb.webSearch["inspire"].retrieveInspireID(tests["inspire"][0]), "")

	def test_inspireoai_other(self):
		"""test auxiliary functions in inspireoai module"""
		marcxmlRecord1 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(metadataPrefix = 'marcxml', identifier = "oai:inspirehep.net:1385583")[1]
		marcxmlRecord2 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(metadataPrefix = 'marcxml', identifier = "oai:inspirehep.net:1414175")[1]
		self.assertEqual(get_journal_ref_xml(marcxmlRecord1),
			(['J.Phys.'], ['G43'], ['2016'], ['033001'], [None], [None], [None], [None]))
		self.assertEqual(get_journal_ref_xml(marcxmlRecord2),
			([None], [None], ['2017'], ['469-475'], [None], [None], [None], ['C15-08-20']))
		dict1a = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord1)
		dict1b = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord1, readConferenceTitle = True)
		self.assertEqual(dict1a, dict1b)
		self.assertEqual(dict1a,
			{'doi': '10.1088/0954-3899/43/3/033001', 'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra', 'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.', 'volume': 'G43', 'year': '2016', 'pages': '033001', 'firstdate': '2015-07-29', 'pubdate': '2016-01-13', 'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.', 'collaboration': None, 'primaryclass': 'hep-ph', 'archiveprefix': 'arXiv', 'eprint': '1507.08204', 'reportnumber': None, 'title': 'Light sterile neutrinos', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'link': '%s10.1088/0954-3899/43/3/033001'%pbConfig.doiUrl, 'bibtex': '@Article{Gariazzo:2015rra,\n        author = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.",\n         title = "{Light sterile neutrinos}",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n archiveprefix = "arXiv",\n  primaryclass = "hep-ph",\n        eprint = "1507.08204",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n'})
		dict2 = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord2, readConferenceTitle = True)
		self.assertEqual(dict2, {'doi': '10.1142/9789813224568_0076', 'arxiv': '1601.01475', 'bibkey': 'Gariazzo:2016ehl', 'ads': None, 'journal': None, 'volume': None, 'year': '2017', 'pages': '469-475', 'firstdate': '2016-01-07', 'pubdate': '2017', 'author': 'Gariazzo, Stefano', 'collaboration': None, 'primaryclass': 'astro-ph.CO', 'archiveprefix': 'arXiv', 'eprint': '1601.01475', 'reportnumber': None, 'title': 'Light Sterile Neutrinos In Cosmology', 'isbn': None, 'booktitle': 'Proceedings, 17th Lomonosov Conference on Elementary Particle Physics: Moscow, Russia, August 20-26, 2015', 'ENTRYTYPE': 'inproceedings', 'oldkeys': '', 'link': '%s10.1142/9789813224568_0076'%pbConfig.doiUrl, 'bibtex': '@Inproceedings{Gariazzo:2016ehl,\n        author = "Gariazzo, Stefano",\n         title = "{Light Sterile Neutrinos In Cosmology}",\n     booktitle = "{Proceedings, 17th Lomonosov Conference on Elementary Particle Physics: Moscow, Russia, August 20-26, 2015}",\n          year = "2017",\n         pages = "469-475",\n archiveprefix = "arXiv",\n  primaryclass = "astro-ph.CO",\n        eprint = "1601.01475",\n           doi = "10.1142/9789813224568_0076",\n}\n\n'})
		dict1a["id"] = "1385583"
		dict2["id"] = "1414175"
		bibtex1 = '@article{abc,\nauthor="me",}'
		bibtex2 = '@article{abc,'
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict2, bibtex1), (False, bibtex1))
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict1a, bibtex2), (False, bibtex2))
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict1a, bibtex1),
			(True, '@Article{abc,\n        author = "me",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n'))

	@unittest.skipIf(skipOAITests, "Online tests with OAI")
	def test_inspireoai(self):
		"""test retrieve daily data from inspireOAI"""
		date1 = (datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d")
		date2 = datetime.date.today().strftime("%Y-%m-%d")
		yren, monen, dayen = date1.split('-')
		yrst, monst, dayst = date2.split('-')
		date1 = datetime.datetime(int(yren), int(monen), int(dayen))
		date2 = datetime.datetime(int(yrst), int(monst), int(dayst))
		result = physBiblioWeb.webSearch["inspireoai"].retrieveOAIUpdates(date1, date2)
		self.assertEqual(type(result), list)
		print(len(result), result[0])

class TestWebImportOffline(unittest.TestCase):
	"""
	Offline test for some functions of the webImport package.
	"""
	def test_arxivYear(self):
		"""test the arxivDaily method in the arxiv module"""
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("abc"), None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("0123.345"), None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("hep-ph/9900000"), "1999")
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("hep-ph/990000"), None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("1234.5678"), "2012")
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("1678.56789"), "2016")

	def test_arxivDaily(self):
		"""test the arxivDaily method in the arxiv module"""
		self.assertFalse(physBiblioWeb.webSearch["arxiv"].arxivDaily("missing"))
		self.assertFalse(physBiblioWeb.webSearch["arxiv"].arxivDaily("physics.missing"))
		content_example='''<?xml version="1.0" encoding="UTF-8"?>

<rdf:RDF
 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 xmlns="http://purl.org/rss/1.0/"
 xmlns:content="http://purl.org/rss/1.0/modules/content/"
 xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
 xmlns:dc="http://purl.org/dc/elements/1.1/"
 xmlns:syn="http://purl.org/rss/1.0/modules/syndication/"
 xmlns:admin="http://webns.net/mvcb/"
>

<channel rdf:about="http://arxiv.org/">
<title>hep-ex updates on arXiv.org</title>
<link>http://arxiv.org/</link>
<description rdf:parseType="Literal">High Energy Physics - Experiment (hep-ex) updates on the arXiv.org e-print archive</description>
<dc:language>en-us</dc:language>
<dc:date>2018-05-23T20:30:00-05:00</dc:date>
<dc:publisher>www-admin@arxiv.org</dc:publisher>
<dc:subject>High Energy Physics - Experiment</dc:subject>
<syn:updateBase>1901-01-01T00:00+00:00</syn:updateBase>
<syn:updateFrequency>1</syn:updateFrequency>
<syn:updatePeriod>daily</syn:updatePeriod>
<items>
 <rdf:Seq>
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.08875" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.09166" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.06430" />
 </rdf:Seq>
</items>
<image rdf:resource="http://arxiv.org/icons/sfx.gif" />
</channel>
<image rdf:about="http://arxiv.org/icons/sfx.gif">
<title>arXiv.org</title>
<url>http://arxiv.org/icons/sfx.gif</url>
<link>http://arxiv.org/</link>
</image>
<item rdf:about="http://arxiv.org/abs/1805.08875">
<title>Measurement of Transverse Single Spin Asymmetries. (arXiv:1805.08875v1 [hep-ex])</title>
<link>http://arxiv.org/abs/1805.08875</link>
<description rdf:parseType="Literal">&lt;p&gt;In 2015 the first collisions between polarized protons and nuclei occurred at
the Relativistic Heavy Ion Collider (RHIC), at a center-of-mass energy of
$\sqrt{s_{NN}}=200$ GeV. Comparisons between spin asymmetries and
cross-sections in $p+p$ production to those in $p+A$ production provide insight
into nuclear structure, namely nuclear modification factors, nuclear dependence
of spin asymmetries, and comparison to models with saturation effects. The
transverse single-spin asymmetry, $A_{N}$, has been measured in $\pi^{0}$
production in the STAR Forward Meson Spectrometer (FMS), an electromagnetic
calorimeter covering a forward psuedorapidity range of $2.6&amp;lt;\eta&amp;lt;4$. Within
this kinematic range, STAR has previously reported the persistence of large
$\pi^0$ asymmetries with unexpected dependences on $p_T$ and event topology in
$p+p$ collisions. This talk will compare these dependences to those in $p+A$
production.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/hep-ex/1/au:+Dilks_C/0/1/0/all/0/1&quot;&gt;Christopher Dilks&lt;/a&gt; (STAR Collaboration)</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.09166">
<title>Recent Top quark results from the Tevatron. (arXiv:1805.09166v1 [hep-ex] CROSS LISTED)</title>
<link>http://arxiv.org/abs/1805.09166</link>
<description rdf:parseType="Literal">&lt;p&gt;We present recent measurements
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/hep-ex/1/au:+Tuchming_B/0/1/0/all/0/1&quot;&gt;Boris Tuchming&lt;/a&gt;</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.06430">
<title>New Perspective on Hybrid Mesons. (arXiv:1805.06430v2 [nucl-th] UPDATED)</title>
<link>http://arxiv.org/abs/1805.06430</link>
<description rdf:parseType="Literal">&lt;p&gt;It is thought that strong interactions within the Standard Model can generate
bound-states in which non-Abelian gauge-bosons play a dual role, serving both
as force and matter fields.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Xu_S/0/1/0/all/0/1&quot;&gt;Shu-Sheng Xu&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Cui_Z/0/1/0/all/0/1&quot;&gt;Zhu-Fang Cui&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Chang_L/0/1/0/all/0/1&quot;&gt;Lei Chang&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Papavassiliou_J/0/1/0/all/0/1&quot;&gt;Joannis Papavassiliou&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Roberts_C/0/1/0/all/0/1&quot;&gt;Craig D. Roberts&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+Zong_H/0/1/0/all/0/1&quot;&gt;Hong-Shi Zong&lt;/a&gt;</dc:creator>
</item>
</rdf:RDF>'''
		with patch('physbiblio.webimport.webInterf.webInterf.textFromUrl', return_value=content_example) as _fromUrl:
			result = physBiblioWeb.webSearch["arxiv"].arxivDaily("hep-ex")
			_fromUrl.assert_called_once_with("https://export.arxiv.org/rss/hep-ex")
			pbConfig.params["maxAuthorNames"] = 3
			self.assertEqual(result[0],
				{'primaryclass': u'hep-ex', 'author': u'Christopher Dilks', 'title': u'Measurement of Transverse Single Spin Asymmetries.', 'abstract': u'In 2015 the first collisions between polarized protons and nuclei occurred at the Relativistic Heavy Ion Collider (RHIC), at a center-of-mass energy of $\\sqrt{s_{NN}}=200$ GeV. Comparisons between spin asymmetries and cross-sections in $p+p$ production to those in $p+A$ production provide insight into nuclear structure, namely nuclear modification factors, nuclear dependence of spin asymmetries, and comparison to models with saturation effects. The transverse single-spin asymmetry, $A_{N}$, has been measured in $\\pi^{0}$ production in the STAR Forward Meson Spectrometer (FMS), an electromagnetic calorimeter covering a forward psuedorapidity range of $2.6&lt;\\eta&lt;4$. Within this kinematic range, STAR has previously reported the persistence of large $\\pi^0$ asymmetries with unexpected dependences on $p_T$ and event topology in $p+p$ collisions. This talk will compare these dependences to those in $p+A$ production. ', 'cross': False, 'version': u'1805.08875v1', 'eprint': u'1805.08875', 'authors': [u'Christopher Dilks'], 'replacement': False})
			self.assertEqual(result[1],
				{'primaryclass': u'hep-ex', 'author': u'Boris Tuchming', 'title': u'Recent Top quark results from the Tevatron.', 'abstract': u'We present recent measurements ', 'cross': True, 'version': u'1805.09166v1', 'eprint': u'1805.09166', 'authors': [u'Boris Tuchming'], 'replacement': False})
			self.assertEqual(result[2],
				{'primaryclass': u'nucl-th', 'author': u'Shu-Sheng Xu and Zhu-Fang Cui and Lei Chang and others', 'title': u'New Perspective on Hybrid Mesons.', 'abstract': u'It is thought that strong interactions within the Standard Model can generate bound-states in which non-Abelian gauge-bosons play a dual role, serving both as force and matter fields. ', 'cross': True, 'version': u'1805.06430v2', 'eprint': u'1805.06430', 'authors': [u'Shu-Sheng Xu', u'Zhu-Fang Cui', u'Lei Chang', u'Joannis Papavassiliou', u'Craig D. Roberts', u'Hong-Shi Zong'], 'replacement': True})

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
import sys
from PySide.QtCore import *
from PySide.QtGui  import *

try:
	from physbiblio.errors import pBLogger
	from physbiblio.database import pBDB, cats_alphabetical
	from physbiblio.config import pbConfig
	from physbiblio.gui.ErrorManager import pBGUILogger
	from physbiblio.gui.DialogWindows import *
	from physbiblio.gui.CommonClasses import *
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside: Run script update_resources.sh")

def editCategory(parent, statusBarObject, editIdCat = None, useParent = None):
	if editIdCat is not None:
		edit = pBDB.cats.getDictByID(editIdCat)
	else:
		edit = None
	newCatWin = editCat(parent, cat = edit, useParent = useParent)
	newCatWin.exec_()
	data = {}
	if newCatWin.result:
		for k, v in newCatWin.textValues.items():
			if k == "parentCat":
				try:
					s = str(newCatWin.selectedCats[0])
				except IndexError:
					s = "0"
			else:
				s = "%s"%v.text()
			data[k] = s
		if data["name"].strip() != "":
			if "idCat" in data.keys():
				print("[GUI] Updating category %s..."%data["idCat"])
				pBDB.cats.update(data, data["idCat"])
			else:
				pBDB.cats.insert(data)
			message = "Category saved"
			statusBarObject.setWindowTitle("PhysBiblio*")
			try:
				parent.recreateTable()
			except:
				pass
		else:
			message = "ERROR: empty category name"
	else:
		message = "No modifications to categories"
	try:
		statusBarObject.StatusBarMessage(message)
	except:
		pass

def deleteCategory(parent, statusBarObject, idCat, name):
	if askYesNo("Do you really want to delete this category (ID = '%s', name = '%s')?"%(idCat, name)):
		pBDB.cats.delete(int(idCat))
		statusBarObject.setWindowTitle("PhysBiblio*")
		message = "Category deleted"
		parent.recreateTable()
	else:
		message = "Nothing changed"
	try:
		statusBarObject.StatusBarMessage(message)
	except:
		pass

class CatsModel(TreeModel):
	def __init__(self, cats, rootElements, parent = None, previous = [], multipleRecords = False):
		self.cats = cats
		self.rootElements = rootElements
		TreeModel.__init__(self)
		self.parentObj = parent
		self.selectedCats = {}
		self.previousSaved = {}
		for cat in self.cats:
			self.selectedCats[cat["idCat"]] = False
			self.previousSaved[cat["idCat"]] = False
		for prevIx in previous:
			try:
				if multipleRecords:
					self.previousSaved[prevIx] = True
					self.selectedCats[prevIx] = "p"
				else:
					self.selectedCats[prevIx] = True
			except IndexError:
				pBLogger.warning("Invalid idCat in previous selection: %s"%prevIx)

	def _getRootNodes(self):
		return [NamedNode(elem, None, index)
			for index, elem in enumerate(self.rootElements)]

	def columnCount(self, parent):
		return 1

	def data(self, index, role):
		if not index.isValid():
			return None
		row = index.row()
		column = index.column()
		value = index.internalPointer().ref.text
		idCat = index.internalPointer().ref.idCat
		if role == Qt.DisplayRole and index.column() == 0:
			return value

		if role == Qt.CheckStateRole and self.parentObj.askCats and column == 0:
			if self.previousSaved[idCat] == True and self.selectedCats[idCat] == "p":
				return Qt.PartiallyChecked
			elif self.selectedCats[idCat] == False:
				return Qt.Unchecked
			else:
				return Qt.Checked
		if role == Qt.EditRole:
			return value
		if role == Qt.DisplayRole:
			return value
		return None

	def flags(self, index):
		if not index.isValid():
			return None
		if index.column() == 0 and self.parentObj.askCats:
			return Qt.ItemIsUserCheckable | Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
		else:
			return Qt.ItemIsEnabled | Qt.ItemIsSelectable

	def headerData(self, section, orientation, role):
		if orientation == Qt.Horizontal and role == Qt.DisplayRole \
			and section == 0:
			return 'Name'
		return None

	def setData(self, index, value, role):
		idCat = index.internalPointer().ref.idCat
		if role == Qt.CheckStateRole and index.column() == 0:
			self.previousSaved[idCat] = False
			if value == Qt.Checked:
				self.selectedCats[idCat] = True
			else:
				self.selectedCats[idCat] = False
		self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),index, index)
		return True

class catsWindowList(QDialog):
	def __init__(self, parent = None, askCats = False, askForBib = None, askForExp = None, expButton = True, previous = [], single = False, multipleRecords = False):
		super(catsWindowList, self).__init__(parent)
		self.parent = parent
		self.setWindowTitle("Categories")
		self.currLayout = QVBoxLayout(self)
		self.askCats = askCats
		self.askForBib = askForBib
		self.askForExp = askForExp
		self.expButton = expButton
		self.previous = previous
		self.single = single
		self.multipleRecords = multipleRecords

		self.setMinimumWidth(400)
		self.setMinimumHeight(600)

		self.fillTree()

	def populateAskCat(self):
		if self.askCats:
			if self.askForBib is not None:
				bibitem = pBDB.bibs.getByBibkey(self.askForBib, saveQuery = False)[0]
				try:
					bibtext = QLabel("Mark categories for the following entry:\n    key:\n%s\n    author(s):\n%s\n    title:\n%s\n"%(self.askForBib, bibitem["author"], bibitem["title"]))
				except:
					bibtext = QLabel("Mark categories for the following entry:\n    key:\n%s\n"%(self.askForBib))
				self.currLayout.addWidget(bibtext)
			elif self.askForExp is not None:
				pass
			else:
				if self.single:
					comment = QLabel("Select the desired category (only the first one will be considered):")
				else:
					comment = QLabel("Select the desired categories:")
				self.currLayout.addWidget(comment)
			self.marked = []
			self.parent.selectedCats = []

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self, exps = False):
		self.parent.selectedCats = [idC for idC in self.root_model.selectedCats.keys() if self.root_model.selectedCats[idC] == True]
		self.parent.previousUnchanged = [idC for idC in self.root_model.previousSaved.keys() if self.root_model.previousSaved[idC] == True]

		if self.single and len(self.parent.selectedCats) > 1 and self.parent.selectedCats[0] == 0:
			self.parent.selectedCats.pop(0)
		if exps:
			self.result	= "Exps"
		else:
			self.result	= "Ok"
		self.close()

	def changeFilter(self, string):
		self.proxyModel.setFilterRegExp(str(string))
		self.tree.expandAll()

	def onAskExps(self):
		self.onOk(exps = True)

	def onNewCat(self):
		editCategory(self.parent, self.parent)
		self.recreateTable()

	def keyPressEvent(self, e):
		if e.key() == Qt.Key_Escape:
			self.close()

	def fillTree(self):
		self.populateAskCat()

		catsTree = pBDB.cats.getHier()

		self.filterInput = QLineEdit("",  self)
		self.filterInput.setPlaceholderText("Filter cateogries")
		self.filterInput.textChanged.connect(self.changeFilter)
		self.currLayout.addWidget(self.filterInput)
		self.filterInput.setFocus()

		self.tree = QTreeView(self)
		self.currLayout.addWidget(self.tree)
		self.tree.setMouseTracking(True)
		self.tree.entered.connect(self.handleItemEntered)

		catsNamedTree = self._populateTree(catsTree[0], 0)

		self.root_model = CatsModel(pBDB.cats.getAll(), [catsNamedTree], self, self.previous, multipleRecords = self.multipleRecords)
		self.proxyModel = LeafFilterProxyModel(self)
		self.proxyModel.setSourceModel(self.root_model)
		self.proxyModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
		self.proxyModel.setSortCaseSensitivity(Qt.CaseInsensitive)
		self.proxyModel.setFilterKeyColumn(-1)
		self.tree.setModel(self.proxyModel)

		self.tree.expandAll()

		self.tree.setHeaderHidden(True)
		#self.tree.doubleClicked.connect(self.askAndPerformAction)

		self.newCatButton = QPushButton('Add new category', self)
		self.newCatButton.clicked.connect(self.onNewCat)
		self.currLayout.addWidget(self.newCatButton)
		
		if self.askCats:
			self.acceptButton = QPushButton('OK', self)
			self.acceptButton.clicked.connect(self.onOk)
			self.currLayout.addWidget(self.acceptButton)
			
			if self.expButton:
				self.expsButton = QPushButton('Ask experiments', self)
				self.expsButton.clicked.connect(self.onAskExps)
				self.currLayout.addWidget(self.expsButton)

			# cancel button
			self.cancelButton = QPushButton('Cancel', self)
			self.cancelButton.clicked.connect(self.onCancel)
			self.cancelButton.setAutoDefault(True)
			self.currLayout.addWidget(self.cancelButton)

	def _populateTree(self, children, idCat):
		name = pBDB.cats.getByID(idCat)[0]["name"]
		children_list = []
		for child in cats_alphabetical(children, pBDB):
			child_item = self._populateTree(children[child], child)
			children_list.append(child_item)
		return NamedElement(idCat, name, children_list)

	def handleItemEntered(self, index):
		if index.isValid():
			row = index.row()
		else:
			return
		idCat, catName = self.proxyModel.sibling(row, 0, index).data().split(": ")
		idCat = idCat.strip()
		try:
			catData = pBDB.cats.getByID(idCat)[0]
		except IndexError:
			pBGUILogger.exception("Failed in finding category")
			return
		QToolTip.showText(
			QCursor.pos(),
			"{idC}: {cat}\nCorresponding entries: {en}\nAssociated experiments: {ex}".format(
				idC = idCat, cat = catData["name"], en = pBDB.catBib.countByCat(idCat), ex = pBDB.catExp.countByCat(idCat)),
			self.tree.viewport(),
			self.tree.visualRect(index)
		)

	def contextMenuEvent(self, event):
		index = self.tree.selectedIndexes()[0]
		row = index.row()
		idCat, catName = self.proxyModel.sibling(row, 0, index).data().split(": ")
		idCat = idCat.strip()
		
		menu = QMenu()
		titAct = menu.addAction("--Category: %s--"%catName).setDisabled(True)
		menu.addSeparator()
		bibAction = menu.addAction("Open list of corresponding entries")
		menu.addSeparator()
		modAction = menu.addAction("Modify")
		delAction = menu.addAction("Delete")
		menu.addSeparator()
		subAction = menu.addAction("Add subcategory")

		action = menu.exec_(event.globalPos())
		if action == bibAction:
			searchDict = {"cats": {"id": [idCat], "operator": "and"}}
			self.parent.reloadMainContent(pBDB.bibs.fetchFromDict(searchDict).lastFetched)
		elif action == modAction:
			editCategory(self, self.parent, idCat)
		elif action == delAction:
			deleteCategory(self, self.parent, idCat, catName)
		elif action == subAction:
			editCategory(self, self.parent, useParent = idCat)

	def cleanLayout(self):
		"""delete previous widgets"""
		while True:
			o = self.layout().takeAt(0)
			if o is None: break
			o.widget().deleteLater()

	def recreateTable(self):
		"""delete previous widgets and create new ones"""
		self.cleanLayout()
		self.fillTree()

class editCat(editObjectWindow):
	"""create a window for editing or creating a category"""
	def __init__(self, parent = None, cat = None, useParent = None):
		super(editCat, self).__init__(parent)
		if cat is None:
			self.data = {}
			for k in pBDB.tableCols["categories"]:
				self.data[k] = ""
		else:
			self.data = cat
		if useParent is not None:
			self.data["parentCat"] = useParent
			self.selectedCats = [useParent]
		elif cat is not None:
			try:
				self.selectedCats = [pBDB.cats.getParent(cat["idCat"])[0][0]]
			except IndexError:
				self.selectedCats = [0]
		else:
			self.selectedCats = [0]
		self.createForm()

	def onAskParent(self):
		selectCats = catsWindowList(parent = self, askCats = True, expButton = False, single = True, previous = self.selectedCats)
		selectCats.exec_()
		if selectCats.result == "Ok":
			try:
				val = self.selectedCats[0]
				self.textValues["parentCat"].setText("%s - %s"%(str(val), pBDB.cats.getByID(val)[0]["name"]))
			except IndexError:
				self.textValues["parentCat"].setText("Select parent")

	def createForm(self):
		self.setWindowTitle('Edit category')

		i = 0
		for k in pBDB.tableCols["categories"]:
			val = self.data[k]
			if k != "idCat" or (k == "idCat" and self.data[k] != ""):
				i += 1
				self.currGrid.addWidget(QLabel(pBDB.descriptions["categories"][k]), i*2-1, 0)
				self.currGrid.addWidget(QLabel("(%s)"%k), i*2-1, 1)
				if k == "parentCat":
					try:
						val = self.selectedCats[0]
						self.textValues[k] = QPushButton("%s - %s"%(str(val), pBDB.cats.getByID(val)[0]["name"]), self)
					except IndexError:
						self.textValues[k] = QPushButton("Select parent", self)
					self.textValues[k].clicked.connect(self.onAskParent)
				else:
					self.textValues[k] = QLineEdit(str(val))
				if k == "idCat":
					self.textValues[k].setEnabled(False)
				self.currGrid.addWidget(self.textValues[k], i*2, 0, 1, 2)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currGrid.addWidget(self.acceptButton, i*2+1, 1)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currGrid.addWidget(self.cancelButton, i*2+1, 0)

		self.setGeometry(100,100,400, 50*i)
		self.centerWindow()
